@@ -74,11 +74,22 @@ void mgls_prepare3v(mglData *ex, mglData *ey, mglData *ez);
 void save(mglGraph *gr,const char *name,const char *suf);
 void test(mglGraph *gr)
 {
+//	gr->Box();	gr->WritePNG("1.png");	return;
+	
 	mglParse par;
-	par.Execute(gr,"new f 100 'x^3':save f 'test.dat':axis:box:fplot ':test.dat:-1:1'");
-	par.Execute(gr,"text 0 0 'ab' '@'");
+	par.Execute(gr,"colorbar;size -2;crange 0 1;value 90");
+// 	par.Execute(gr,"fplot 'x';legend 'рус':legend");
+//	par.Execute(gr,"fsurf 'x*y':rasterize:clf:fplot 'sin(x)' '2q':axis");
+//	par.Execute(gr,"xrange 0 1:new x 10 '3*x':echo x\nnew a 10 'value([0,1,-1],3*x)':echo a\nnew b 10 'spline([0,1,-1],3*x)':echo b");
+//	par.Execute(gr,"new a 1 'sum(_i^2,5)':echo a:new b 1 'prod(1+_i,5)':echo b\n"
+//	"new c 1 'dsum(_i,5)':echo c:new d 1 'sum(sum(_j+_i^2,5),5)':echo d");
+//	par.Execute(gr,"define n 30\nnew a n n n 'x^2+y^2'\nnew b n n n 'z^4+2*x^4-2*y^4'\nrotate 40 60:box:axis\n"
+//		"dcont [0.5,0.5] a b '2k'\nlight on:alpha on\nsurf3 a 0.5 'r':surf3 b 0.5 'g'");
+//	par.Execute(gr,"new f 100 'x^3':save f 'test.dat':axis:box:fplot ':test.dat:-1:1'");
+//	par.Execute(gr,"text 0 0 'ab' '@'");
 //	par.Execute(gr,"new a 3 3 'x*y'\nsubplot 2 1 0 '':dens a:dens a 'k+/'\n"
 //	"subplot 2 1 1 '':mask '+' 'ff00182424f800' 30:dens a '3+'");
+	gr->WritePNG("1.png");
 	return;
 	
 	dual c(0,M_PI/2), r=mgl_ipowc(c,2);
@@ -152,6 +163,18 @@ void mgl_generate_texi()
 	fclose(fs);	fclose(fp);	fclose(fq);
 }
 
+void mgl_generate_slides()
+{
+	FILE *fp = fopen("samples.tex","w");
+	FILE *fs = fopen("samples.cpp","r");
+	for(size_t i=0;samp[i].name && samp[i].name[0];i++)
+	{
+		fprintf(fp,"\n\\begin{frame}[fragile]{Пример \\href{http://mathgl.sourceforge.net/doc_en/%s-sample.html}{\\texttt{%s}}}\n%s\n", samp[i].name, samp[i].name, samp[i].info);
+		fprintf(fp,"\\begin{center}\n\\includegraphics[width=0.7\\linewidth]{png/%s}\n\\end{center}\n\\end{frame}\n", samp[i].name);
+	}
+	fclose(fp);	fclose(fs);
+}
+
 static struct option longopts[] =
 {
 	{ "big",	no_argument,	&big,		1 },
@@ -343,7 +366,7 @@ int main(int argc,char **argv)
 			default:	usage();	return 0;
 		}
 
-	if(dotest==1)	printf("Global (before):%s\n",mglGlobalMess.c_str());
+	if(dotest==1)	printf("Global (before):%s\n",mgl_get_global_warn());
 	gr = new mglGraph;
 	if(	type==11|| type==12|| type==5 || type==9)	width=height;
 	switch(big)
@@ -367,7 +390,7 @@ int main(int argc,char **argv)
 		gr->WriteSVG("test.svg");
 		gr->WriteEPS("test.eps");
 		printf("Messages:%s\n",gr->Message());
-		printf("Global:%s\n",mglGlobalMess.c_str());
+		printf("Global:%s\n",mgl_get_global_warn());
 		delete gr;	return 0;
 	}
 	else if(dotest==2)	// NOTE mgl_gen_fnt[###][6] have to be updated if new glyphs will be added to built-in font
@@ -396,6 +419,7 @@ int main(int argc,char **argv)
 		clock_t beg,end;
 		while(s->name[0])	// all samples
 		{
+			if(!strcmp(s->name,"icon"))	{	s++;	continue;	}
 			char *buf = new char[strlen(s->mgl)+ll];
 			strcpy(buf,s->mgl);	strcat(buf,mmgl_dat_prepare);
 			fprintf(fp,"\n@item %s",s->name);
@@ -410,7 +434,7 @@ int main(int argc,char **argv)
 				else 	par.Execute(gr,buf);
 				gr->Finish();
 				end = clock();
-				fprintf(fp," @tab %.3g",double(end-beg)/CLOCKS_PER_SEC);
+				fprintf(fp," @tab %.2g",double(end-beg)/CLOCKS_PER_SEC);
 				printf("\t%d->%g",qual[i],double(end-beg)/CLOCKS_PER_SEC);
 				fflush(fp);	fflush(stdout);
 			}
@@ -427,6 +451,7 @@ int main(int argc,char **argv)
 	}
 	else if(dotest==6)
 	{
+		mgl_generate_slides();
 		mgl_generate_texi();
 		delete gr;	return 0;
 	}
@@ -439,6 +464,7 @@ int main(int argc,char **argv)
 	{
 		while(s->name[0])	// all samples
 		{
+			if(!strcmp(s->name,"icon"))	{	s++;	continue;	}
 			gr->DefaultPlotParam();	gr->Clf();
 			if(use_mgl)
 			{
