@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -46,8 +46,10 @@ std::string MGL_EXPORT mgl_data_to_string(HCDT d, long ns);
 std::string MGL_EXPORT mgl_datac_to_string(HCDT d, long ns);
 /// Get section separated by symbol ch. This is analog of QString::section().
 std::string MGL_EXPORT mgl_str_arg(const std::string &str, char ch, int n1, int n2=-1);
+std::wstring MGL_EXPORT mgl_wcs_arg(const std::wstring &str, wchar_t ch, int n1, int n2);
 /// Get sections separated by symbol ch
 std::vector<std::string> MGL_EXPORT mgl_str_args(const std::string &str, char ch);
+std::vector<std::wstring> MGL_EXPORT mgl_wcs_args(const std::wstring &str, wchar_t ch);
 /// Get string from real number
 std::string MGL_EXPORT mgl_str_num(double val);
 /// Get string from complex number
@@ -106,6 +108,12 @@ void MGL_EXPORT mgl_data_export_(uintptr_t *dat, const char *fname, const char *
 /// Save data to HDF file
 void MGL_EXPORT mgl_data_save_hdf(HCDT d,const char *fname,const char *data,int rewrite);
 void MGL_EXPORT mgl_data_save_hdf_(uintptr_t *d, const char *fname, const char *data, int *rewrite,int l,int n);
+/// Save value to HDF file
+void MGL_EXPORT mgl_dual_save_hdf(mdual val,const char *fname,const char *data,int rewrite);
+void MGL_EXPORT mgl_real_save_hdf(double val,const char *fname,const char *data,int rewrite);
+void MGL_EXPORT mgl_int_save_hdf(long val,const char *fname,const char *data,int rewrite);
+void MGL_EXPORT mgl_real_save_hdf_(double *val,const char *fname,const char *data,int *rewrite,int,int);
+void MGL_EXPORT mgl_int_save_hdf_(long *val, const char *fname, const char *data, int *rewrite, int, int);
 /// Get information about the data (sizes and momentum) to string
 MGL_EXPORT const char *mgl_data_info(HCDT dat);
 int MGL_EXPORT mgl_data_info_(uintptr_t *dat, char *out, int len);
@@ -226,6 +234,9 @@ struct MGL_EXPORT mglNum
 	{	d=n.d;	c=n.c;	s=n.s;	return n;	}
 };
 //-----------------------------------------------------------------------------
+/// List of user-defined data arrays
+MGL_EXPORT extern std::vector<mglDataA*> mglDataList;
+//-----------------------------------------------------------------------------
 /// Abstract class for data array
 class MGL_EXPORT mglDataA
 {
@@ -237,8 +248,14 @@ public:
 	void (*func)(void *);	///< Callback function for destroying
 	void *o; 		///< Pointer to external object
 
-	mglDataA()	{	temp=false;	func=0;	o=0;	}
-	virtual ~mglDataA()	{	if(func)	func(o);	}
+	mglDataA()	{	mgl_init();	mglDataList.push_back(this);	temp=false;	func=0;	o=0;	}
+	virtual ~mglDataA()
+	{
+		for(long i = mglDataList.size()-1; i>=0; i--)
+			if(mglDataList[i] == this)
+			{	mglDataList.erase(mglDataList.begin()+i);	break;	}
+		if(func)	func(o);
+	}
 	/// Set name for data variable (can be used in mgl_formula_calc() or in MGL scripts)
 	inline void Name(const char *name)		{	s = name;	}
 	inline void Name(const wchar_t *name)	{	s = name;	}
@@ -384,7 +401,7 @@ struct MGL_EXPORT mglColorID
 	mglColor col;
 };
 MGL_EXPORT extern mglColorID mglColorIds[31];
-MGL_EXPORT extern std::string mglGlobalMess;	///< Buffer for receiving global messages
+// MGL_EXPORT extern std::string mglGlobalMess;	///< Buffer for receiving global messages
 //-----------------------------------------------------------------------------
 #endif
 
