@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -267,7 +267,8 @@ struct MGL_EXPORT mglGlyph
 	mglGlyph(long Nt, long Nl):nt(0),nl(0),trig(0),line(0)	{	Create(Nt,Nl);	}
 	~mglGlyph()	{	if(trig)	delete []trig;	if(line)	delete []line;	}
 
-	void Create(long Nt, long Nl);
+	void Create(long Nt, long Nl);	///< Allocate memory for given sizes
+	void Load(wchar_t id, const char *fname);	///< Load glyph 'id' from TTF/OTF file 'fname'
 	bool operator==(const mglGlyph &g) const MGL_FUNC_PURE;
 	inline bool operator!=(const mglGlyph &g) const MGL_FUNC_PURE
 	{	return !(*this==g);	}
@@ -383,11 +384,11 @@ public:
 	inline void set(bool v,uint32_t fl)	{	Flag = v ? Flag|fl : Flag&(~fl);	}
 
 	/// Set axis range scaling -- simplified way to shift/zoom axis range -- need to replot whole image!
-	inline void ZoomAxis(mglPoint p1=mglPoint(0,0,0,0), mglPoint p2=mglPoint(1,1,1,1))	{	AMin = p1;	AMax = p2;	}
+	inline void ZoomAxis(const mglPoint &p1=mglPoint(0,0,0,0), const mglPoint &p2=mglPoint(1,1,1,1))	{	AMin = p1;	AMax = p2;	}
 	/// Set values of mglGraph::Min and mglGraph::Max
 	inline void SetRanges(mreal x1, mreal x2, mreal y1, mreal y2, mreal z1=0, mreal z2=0, mreal c1=0, mreal c2=0)
 	{	SetRanges(mglPoint(x1,y1,z1,c1),mglPoint(x2,y2,z2,c2));	}
-	void SetRanges(mglPoint v1, mglPoint v2);
+	void SetRanges(const mglPoint &v1, const mglPoint &v2);
 	/// Set values of mglGraph::Cmin and mglGraph::Cmax as minimal and maximal values of data a
 	void CRange(HCDT a, bool add = false, mreal fact=0);
 	void CRange(mreal v1,mreal v2,bool add=false);
@@ -425,7 +426,7 @@ public:
 	/// Set additional cutting box
 	inline void SetCutBox(mreal x1, mreal y1, mreal z1, mreal x2, mreal y2, mreal z2)
 	{	CutMin.Set(x1,y1,z1);	CutMax.Set(x2,y2,z2);	}
-	inline void SetCutBox(mglPoint v1, mglPoint v2)	{	CutMin=v1;	CutMax=v2;	}
+	inline void SetCutBox(const mglPoint &v1, const mglPoint &v2)	{	CutMin=v1;	CutMax=v2;	}
 	/// Reset mask to solid state
 	inline void ResetMask()	{	mask = MGL_SOLID_MASK;	MaskAn = DefMaskAn;	}
 	/// Set default mask rotation angle
@@ -495,6 +496,7 @@ public:
 	/// Copy font from another mglGraph instance
 	void CopyFont(mglBase *gr);
 	/// Set default font size
+	void SetFontHscale(mreal val);
 	inline void SetFontSize(mreal val)	{	FontSize=val>0 ? val:-FontSize*val;	}
 	inline mreal GetFontSize() const	{	return FontSize;	}
 	mreal TextWidth(const char *text, const char *font, mreal size) const MGL_FUNC_PURE;
@@ -531,15 +533,15 @@ public:
 
 	// ~~~~~~~~~~~~~~~~~~~~~~ Developer functions ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 	/// Add point to the Pnt and return its position
-	inline long AddPnt(mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1)
+	inline long AddPnt(const mglPoint &p, mreal c=-1, const mglPoint &n=mglPoint(NAN), mreal a=-1, int scl=1)
 	{	return AddPnt(&B,p,c,n,a,scl);	}
-	long AddPnt(const mglMatrix *M, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1);
+	long AddPnt(const mglMatrix *M, const mglPoint &p, mreal c=-1, const mglPoint &n=mglPoint(NAN), mreal a=-1, int scl=1);
 	bool AddPntQ(mglPnt &q, const mglMatrix *M, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1);
-	inline bool AddPntQ(mglPnt &q, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1)
+	inline bool AddPntQ(mglPnt &q, mglPoint p, mreal c=-1, const mglPoint &n=mglPoint(NAN), mreal a=-1, int scl=1)
 	{	return AddPntQ(q,&B,p,c,n,a,scl);	}
-	inline bool AddPntQ(long id, const mglMatrix *M, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1)
+	inline bool AddPntQ(long id, const mglMatrix *M, const mglPoint &p, mreal c=-1, const mglPoint &n=mglPoint(NAN), mreal a=-1, int scl=1)
 	{	return AddPntQ(Pnt[id],M,p,c,n,a,scl);	}
-	inline bool AddPntQ(long id, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1)
+	inline bool AddPntQ(long id, const mglPoint &p, mreal c=-1, const mglPoint &n=mglPoint(NAN), mreal a=-1, int scl=1)
 	{	return AddPntQ(Pnt[id],&B,p,c,n,a,scl);	}
 	inline void SetPntOff(size_t id)	{	Pnt[id].x=NAN;	}
 	long AllocPnts(size_t num);
@@ -548,9 +550,9 @@ public:
 	bool CopyNtoC(mglPnt &q, long k, mreal c);
 	inline bool CopyNtoC(long id, long k, mreal c)
 	{	return (id>=0)?CopyNtoC(Pnt[id],k,c):false;	}
-	long CopyProj(long from, mglPoint p, mglPoint n, short sub=0);
-	bool CopyProj(mglPnt &q, long from, mglPoint p, mglPoint n, short sub=0);
-	void CopyProj(long id, long from, mglPoint p, mglPoint n, short sub=0)
+	long CopyProj(long from, const mglPoint &p, const mglPoint &n, short sub=0);
+	bool CopyProj(mglPnt &q, long from, const mglPoint &p, const mglPoint &n, short sub=0);
+	void CopyProj(long id, long from, const mglPoint &p, const mglPoint &n, short sub=0)
 	{	if(id>=0)	CopyProj(Pnt[id],from,p,n,sub);	}
 	void DisablePnt(long id)	{	Pnt[id].x = NAN;	}
 	void SetRGBA(long k, const mglColor &c)
@@ -655,6 +657,7 @@ public:
 	void SetEventFunc(void (*func)(void *), void *par)	{	event_cb=func;	event_par=par;	}
 
 protected:
+	bool limit_pm1;		///< limit coordinates in range [-1,+1] for OpenGL only
 	volatile bool Stop;	///< Flag that execution should be terminated.
 	void (*event_cb)(void *);	///< Function to be called for event processing
 	void *event_par;	///< Parameter for event processing function
