@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -204,8 +204,16 @@ using mglBase::Light;
 
 	/// Rasterize current plot and set it as background image
 	void Rasterize();
-	/// Load image for background from file
+	/// Load image for background from file (basic variant)
 	void LoadBackground(const char *fname, double alpha=1);
+	/// Load image for background from file. 
+	/** Parameter 'how' can be:
+	 *   'a' for filling current subplot only;
+	 *   's' for spline-based resizing;
+	 *   'f' for fixing aspect ratio at resizing;
+	 *   'c' for centering image;
+	 *   't' for tessellate image. */
+	void LoadBackground(const char *fname, const char *how, double alpha=1);
 	/// Fill background image by specified color
 	void FillBackground(const mglColor &cc);
 
@@ -216,7 +224,7 @@ using mglBase::Light;
 	mglPoint CalcXYZ(int xs, int ys, bool real=false) const MGL_FUNC_PURE;
 	/// Calculate screen point {xs,ys} for 3D coordinate {x,y,z}
 	void CalcScr(mglPoint p, int *xs, int *ys) const;
-	mglPoint CalcScr(mglPoint p) const;
+	mglPoint CalcScr(const mglPoint &p) const;
 	/// Set object/subplot id
 	inline void SetObjId(long id)	{	ObjId = id;	}
 	/// Get object id
@@ -291,7 +299,7 @@ using mglBase::Light;
 	/// Set the ticks parameters
 	void SetTicks(char dir, mreal d=0, int ns=0, mreal org=NAN, const wchar_t *lbl=0);
 	/// Auto adjust ticks
-	void AdjustTicks(const char *dir="xyzc", bool force=false, std::string stl="");
+	void AdjustTicks(const char *dir="xyzc", bool force=false, const std::string &stl="");
 	/// Tune ticks
 	inline void SetTuneTicks(int tune, mreal pos=1.15)
 	{	TuneTicks = tune;	FactorPos = pos;	}
