@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -31,6 +31,9 @@ uintptr_t MGL_EXPORT mgl_create_graph_(int *width, int *height);
 /// Delete HMGL object
 void MGL_EXPORT mgl_delete_graph(HMGL gr);
 void MGL_EXPORT mgl_delete_graph_(uintptr_t *gr);
+/// Return pointer to built-in (default) HMGL object
+HMGL MGL_EXPORT mgl_default_graph();
+uintptr_t MGL_EXPORT mgl_default_graph_();
 /// Set size of frame in pixels. Normally this function is called internally.
 void MGL_EXPORT mgl_set_size(HMGL gr, int width, int height);
 void MGL_EXPORT mgl_set_size_(uintptr_t *gr, int *width, int *height);
@@ -175,7 +178,7 @@ void MGL_EXPORT mgl_colorbar_(uintptr_t *gr, const char *sch,int);
  *	 ‘-’ for printing usual ‘-’ in ticks labels;
  *	 ‘0123456789’ for precision at printing ticks labels.*/
 void MGL_EXPORT mgl_colorbar_ext(HMGL gr, const char *sch, double x, double y, double w, double h);
-void MGL_EXPORT mgl_colorbar_ext_(uintptr_t *gr, const char *sch, mreal *x, mreal *y, mreal *w, mreal *h, int);
+void MGL_EXPORT mgl_colorbar_ext_(uintptr_t *gr, const char *sch, mreal *x, mreal *y, mreal *w, mreal *h,int);
 /// Draw colorbar with manual colors at edge of axis
 /** Parameter \a sch may contain:
  *	 ‘<>^_’ for positioning at left, at right, at top or at bottom correspondingly;
@@ -205,7 +208,7 @@ void MGL_EXPORT mgl_colorbar_val_(uintptr_t *gr, uintptr_t *dat, const char *sch
  *	 ‘-’ for printing usual ‘-’ in ticks labels;
  *	 ‘0123456789’ for precision at printing ticks labels.*/
 void MGL_EXPORT mgl_colorbar_val_ext(HMGL gr, HCDT dat, const char *sch,double x, double y, double w, double h);
-void MGL_EXPORT mgl_colorbar_val_ext_(uintptr_t *gr, uintptr_t *dat, const char *sch, mreal *x, mreal *y, mreal *w, mreal *h, int);
+void MGL_EXPORT mgl_colorbar_val_ext_(uintptr_t *gr, uintptr_t *dat, const char *sch, mreal *x, mreal *y, mreal *w, mreal *h,int);
 
 /// Add string to legend
 void MGL_EXPORT mgl_add_legend(HMGL gr, const char *text,const char *style);
@@ -444,6 +447,17 @@ void MGL_EXPORT mgl_clf_str_(uintptr_t *gr, const char *col, int);
 /// Load background image
 void MGL_EXPORT mgl_load_background(HMGL gr, const char *fname, double alpha);
 void MGL_EXPORT mgl_load_background_(uintptr_t *gr, const char *fname, mreal *alpha, int);
+/// Load background image
+/** Parameter 'how' can be:
+	*   'a' for filling current subplot only;
+	*   's' for scaling (resizing) image to whole area;
+	*   'c' for centering image;
+	*   'm' for tessellate image as mosaic. */
+void MGL_EXPORT mgl_load_background_ext(HMGL gr, const char *fname, const char *how, double alpha);
+void MGL_EXPORT mgl_load_background_ext_(uintptr_t *gr, const char *fname, const char *how, mreal *alpha, int,int);
+/// Fill background by specified color. Colors r,g,b should be in range [0,1].
+void MGL_EXPORT mgl_fill_background(HMGL gr, double r, double g, double b);
+void MGL_EXPORT mgl_fill_background_(uintptr_t *gr, double *r, double *g, double *b);
 
 /// Put further plotting in m-th cell of nx*ny grid of the image.
 /** String \a style may contain:
