@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -61,6 +61,8 @@
 #if MGL_HAVE_ATTRIBUTE
 #define MGL_FUNC_CONST	__attribute__((const))
 #define MGL_FUNC_PURE	__attribute__((pure))
+#define MGL_FUNC_INIT	__attribute__((constructor))
+#define MGL_FUNC_FINI	__attribute__((destructor))
 #else
 #define MGL_FUNC_CONST
 #define MGL_FUNC_PURE
@@ -146,7 +148,6 @@ typedef size_t msize;
 
 #if !MGL_SYS_NAN
 #include <float.h>
-#include <math.h>
 const unsigned long long mgl_nan[2] = {0x7fffffffffffffff, 0x7fffffff};
 const unsigned long long mgl_inf[2] = {0x7ff0000000000000, 0x7f800000};
 #define NANd    (*(double*)mgl_nan)
@@ -293,6 +294,7 @@ extern MGL_EXPORT uint64_t mgl_mask_val[16];
 #include <vector>
 #include <complex>
 typedef std::complex<mreal> dual;
+typedef std::complex<double> ddual;
 //-----------------------------------------------------------------------------
 inline bool mgl_isrange(double a, double b)
 {	return fabs(a-b)>MGL_MIN_VAL && a-a==0. && b-b==0.;	}
@@ -351,7 +353,9 @@ typedef cmdual mdual;
 #endif
 #endif
 //-----------------------------------------------------------------------------
-extern float mgl_cos[360];	///< contain cosine with step 1 degree
+extern float MGL_EXPORT mgl_cos[360];	///< contain cosine with step 1 degree
+void MGL_EXPORT MGL_FUNC_INIT mgl_init();	///< initialize MathGL structures
+void MGL_EXPORT MGL_FUNC_FINI mgl_fini();	///< free MathGL structures
 //-----------------------------------------------------------------------------
 /// Calculate sqrt(x*x+y*y)
 double MGL_EXPORT_CONST mgl_hypot(double x, double y);
@@ -388,6 +392,9 @@ void MGL_EXPORT mgl_set_global_warn_(const char *text,int);
 /// Get text of global warning message(s)
 MGL_EXPORT_PURE const char *mgl_get_global_warn();
 int MGL_EXPORT mgl_get_global_warn_(char *out, int len);
+/// Clear global warning message
+void MGL_EXPORT mgl_clear_global_warn();
+void MGL_EXPORT mgl_clear_global_warn_();
 /// Setup gettext usage. NOTE: Russian translation MUST be installed.
 void MGL_EXPORT mgl_textdomain(const char *argv0, const char *locale);
 void MGL_EXPORT mgl_textdomain_(const char *locale, int);
