@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -278,7 +278,7 @@ void mglCanvas::SetTickTime(char dir, mreal d, const char *t)
 	}
 }
 //-----------------------------------------------------------------------------
-void mglCanvas::AdjustTicks(const char *dir, bool force, std::string stl)
+void mglCanvas::AdjustTicks(const char *dir, bool force, const std::string &stl)
 {
 	if(force)	SetTuneTicks(3);
 	UpdateAxis();
@@ -483,7 +483,6 @@ void mglCanvas::Axis(const char *dir, const char *stl, const char *opt)
 
 	mreal angl = SaveState(opt);
 	AdjustTicks(dir,mglchr(stl,'a'),Tstl);
-	LoadState();
 
 	ax.pos = strchr(dir,'X') ? 'T':'t';
 	ay.pos = strchr(dir,'Y') ? 'T':'t';
@@ -505,6 +504,7 @@ void mglCanvas::Axis(const char *dir, const char *stl, const char *opt)
 	else if(strchr(dir,'Y') || strchr(dir,'y'))
 	{	ay.inv = inv;	DrawAxis(ay, text, arr, stl, angl);	}
 	set(ret, MGL_ENABLE_RTEXT);
+	LoadState();
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::DrawAxis(mglAxis &aa, int text, char arr,const char *stl,mreal angl)
@@ -1056,6 +1056,7 @@ void mglCanvas::Colorbar(HCDT v, const char *sch, mreal x, mreal y, mreal w, mre
 void mglCanvas::colorbar(HCDT vv, const mreal *c, int where, mreal x, mreal y, mreal w, mreal h, bool text)
 {
 	static int cgid=1;	StartGroup("Colorbar",cgid++);
+
 	long n=vv->GetNx();
 	mreal s3=B.pf,ss=1/s3;		// NOTE: colorbar was wider ss=0.9;
 	mglPoint p1,p2;
