@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -20,6 +20,14 @@
 #include "mgl2/font.h"
 #include "mgl2/base.h"
 #include "mgl2/eval.h"
+
+#if MGL_HAVE_FREETYPE
+#include <ft2build.h>
+#include FT_FREETYPE_H
+#include FT_OUTLINE_H
+#include FT_BBOX_H
+#endif
+
 #if MGL_HAVE_OMP
 #include <omp.h>
 #endif
@@ -121,7 +129,9 @@ void MGL_EXPORT mgl_strlwr(char *str)
 //-----------------------------------------------------------------------------
 mglBase::mglBase()
 {
+	mgl_init();
 	Flag=0;	saved=false;	PrmInd=NULL;
+	limit_pm1 = false;
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_init(&mutexPnt,0);
 	pthread_mutex_init(&mutexTxt,0);
@@ -187,6 +197,7 @@ mglBase::~mglBase()
 #endif
 }
 //-----------------------------------------------------------------------------
+void mglBase::SetFontHscale(mreal val)	{	fnt->HeightScale(val);	}
 void mglBase::RestoreFont()	{	fnt->Restore();	}
 void mglBase::LoadFont(const char *name, const char *path)
 {	if(name && *name)	fnt->Load(name,path);	else	fnt->Restore();	}
@@ -269,6 +280,117 @@ void mglBase::SetWarn(int code, const char *who)
 //-----------------------------------------------------------------------------
 //		Add glyph to the buffer
 //-----------------------------------------------------------------------------
+#if MGL_HAVE_FREETYPE
+MGL_NO_EXPORT int mgl_glf_moveto(const FT_Vector *to, void *user)
+{
+	std::vector<double> *a = (std::vector<double> *)user;
+	if(a->size()>1)	{	a->push_back(NAN);	a->push_back(NAN);	}
+	a->push_back(to->x);	a->push_back(to->y);
+	return 0;
+}
+MGL_NO_EXPORT int mgl_glf_lineto(const FT_Vector *to, void *user)
+{
+	std::vector<double> *a = (std::vector<double> *)user;
+	if(a->size()<2)	{	a->push_back(0);	a->push_back(0);	}
+	a->push_back(to->x);	a->push_back(to->y);
+	return 0;
+}
+MGL_NO_EXPORT int mgl_glf_parabto(const FT_Vector *ctrl, const FT_Vector *to, void *user)
+{
+	std::vector<double> *a = (std::vector<double> *)user;
+	int x0=0,y0=0, x1=ctrl->x,y1=ctrl->y, x2=to->x,y2=to->y;
+	size_t n=a->size();
+	if(n<2)	{	a->push_back(0);	a->push_back(0);	}
+	else	{	x0 = (*a)[n-2];	y0 = (*a)[n-1];	}
+	for(int i=1;i<=10;i++)
+	{
+		double t = 0.1*i;
+		a->push_back(x0*(1-t)*(1-t)+2*(1-t)*t*x1+t*t*x2);
+		a->push_back(y0*(1-t)*(1-t)+2*(1-t)*t*y1+t*t*y2);
+	}
+	return 0;
+}
+MGL_NO_EXPORT int mgl_glf_cubeto(const FT_Vector *ctrl1, const FT_Vector *ctrl2, const FT_Vector *to, void *user)
+{
+	std::vector<double> *a = (std::vector<double> *)user;
+	int x0=0,y0=0, x1=ctrl1->x,y1=ctrl1->y, x2=ctrl2->x,y2=ctrl2->y, x3=to->x,y3=to->y;
+	size_t n=a->size();
+	if(n<2)	{	a->push_back(0);	a->push_back(0);	}
+	else	{	x0 = (*a)[n-2];	y0 = (*a)[n-1];	}
+	for(int i=1;i<=30;i++)
+	{
+		double t = 0.1*i;
+		a->push_back(x0*(1-t)*(1-t)*(1-t) + 3*(1-t)*t*(x1*(1-t)+t*x2) + t*t*t*x3);
+		a->push_back(y0*(1-t)*(1-t)*(1-t) + 3*(1-t)*t*(y1*(1-t)+t*y2) + t*t*t*y3);
+	}
+	return 0;
+}
+#endif
+void mglGlyph::Load(wchar_t id, const char *fname)
+{
+#if MGL_HAVE_FREETYPE
+	FT_Library m_ftLibrary;
+	if(FT_Init_FreeType(&m_ftLibrary))
+		mgl_set_global_warn("Couldn't initialize the FreeType library.");
+	else
+	{
+		FT_Face m_face;
+		// For simplicity, always use the first face index.
+		if(FT_New_Face(m_ftLibrary, fname, 0, &m_face))
+			mgl_set_global_warn("Couldn't load the font file.");
+		else
+		{
+			// For simplicity, use the charmap FreeType provides by default;
+			// in most cases this means Unicode.
+			FT_UInt index = FT_Get_Char_Index(m_face, id);
+			if(FT_Load_Glyph(m_face, index, FT_LOAD_NO_SCALE|FT_LOAD_NO_BITMAP))
+				mgl_set_global_warn("Couldn't load the glyph.");
+			else
+			{
+				FT_GlyphSlot slot = m_face->glyph;
+				FT_Outline &outline = slot->outline;
+
+				if(slot->format!=FT_GLYPH_FORMAT_OUTLINE || outline.n_contours <= 0 || outline.n_points <= 0 ||FT_Outline_Check(&outline))
+					mgl_set_global_warn("Outline doesn't exist.");
+				else
+				{
+					const FT_Fixed multiplier = 32768L;
+					FT_Matrix matrix;
+					matrix.xx = matrix.yy = multiplier;
+					matrix.xy = matrix.yx = 0;
+					FT_Outline_Transform(&outline, &matrix);
+					
+					FT_Outline_Funcs callbacks;
+					callbacks.move_to = mgl_glf_moveto;
+					callbacks.line_to = mgl_glf_lineto;
+					callbacks.conic_to = mgl_glf_parabto;
+					callbacks.cubic_to = mgl_glf_cubeto;
+					callbacks.shift = 0;
+					callbacks.delta = 0;
+					std::vector<double> xy_coor;
+					if(FT_Outline_Decompose(&outline, &callbacks, &xy_coor))
+						mgl_set_global_warn("Couldn't extract the outline.");
+					nt = -id;	// TODO optimize and copy points. Q: actual width? Q: cmp with known.
+					FT_BBox boundingBox;
+					FT_Outline_Get_BBox(&outline, &boundingBox);
+					FT_Pos xMin = boundingBox.xMin;
+					FT_Pos yMin = boundingBox.yMin;
+					FT_Pos xMax = boundingBox.xMax;
+					FT_Pos yMax = boundingBox.yMax;
+/*					m_xMin = xMin;
+					m_yMin = yMin;
+					m_width = xMax - xMin;
+					m_height = yMax - yMin;*/
+
+				}
+			}
+			FT_Done_Face(m_face);
+		}
+	}
+	FT_Done_FreeType(m_ftLibrary);
+#endif
+}
+//-----------------------------------------------------------------------------
 void mglGlyph::Create(long Nt, long Nl)
 {
 //	if(Nt<0 || Nl<0)	return;
@@ -374,7 +496,7 @@ static void mgl_coor_box(HMGL gr, mglPoint &p)
 	mgl_put_inbox(gr->Min.y, gr->Max.y, p.y);
 	mgl_put_inbox(gr->Min.z, gr->Max.z, p.z);
 }
-long mglBase::AddPnt(const mglMatrix *mat, mglPoint p, mreal c, mglPoint n, mreal a, int scl)
+long mglBase::AddPnt(const mglMatrix *mat, const mglPoint &p, mreal c, const mglPoint &n, mreal a, int scl)
 {
 	mglPnt q;
 	if(!AddPntQ(q,mat,p,c,n,a,scl))	return -1;
@@ -455,7 +577,7 @@ bool mglBase::CopyNtoC(mglPnt &q, long from, mreal c)
 	return mgl_isnum(q.x);
 }
 //-----------------------------------------------------------------------------
-long mglBase::CopyProj(long from, mglPoint p, mglPoint n, short sub)
+long mglBase::CopyProj(long from, const mglPoint &p, const mglPoint &n, short sub)
 {
 	mglPnt q;
 	if(!CopyProj(q,from,p,n,sub))	return -1;
@@ -464,7 +586,7 @@ long mglBase::CopyProj(long from, mglPoint p, mglPoint n, short sub)
 	{k=Pnt.size();	MGL_PUSH(Pnt,q,mutexPnt);}	return k;
 }
 //-----------------------------------------------------------------------------
-bool mglBase::CopyProj(mglPnt &q, long from, mglPoint p, mglPoint n, short sub)
+bool mglBase::CopyProj(mglPnt &q, long from, const mglPoint &p, const mglPoint &n, short sub)
 {
 	if(from<0)	return false;
 	q=Pnt[from];	q.sub = sub;
@@ -659,6 +781,12 @@ bool mglBase::ScalePoint(const mglMatrix *, mglPoint &p, mglPoint &n, bool use_n
 	if(fabs(x)>MGL_FEPSILON || fabs(y)>MGL_FEPSILON || fabs(z)>MGL_FEPSILON)	res = false;
 
 	if(!res && use_nan)	x = NAN;	// extra sign that point shouldn't be plotted
+	else if(limit_pm1)
+	{
+		x = x>1?1:(x<-1?-1:x);
+		y = y>1?1:(y<-1?-1:y);
+		z = z>1?1:(z<-1?-1:z);
+	}
 	return res;
 }
 //-----------------------------------------------------------------------------
@@ -693,7 +821,7 @@ void mglBase::SetOrigin(mreal x0, mreal y0, mreal z0, mreal c0)
 	}
 }
 //-----------------------------------------------------------------------------
-void mglBase::SetRanges(mglPoint m1, mglPoint m2)
+void mglBase::SetRanges(const mglPoint &m1, const mglPoint &m2)
 {
 	if(mgl_isrange(m1.x, m2.x))	{	Min.x=m1.x;	Max.x=m2.x;	}
 	if(mgl_isrange(m1.y, m2.y))	{	Min.y=m1.y;	Max.y=m2.y;	}
@@ -1431,7 +1559,7 @@ mreal mglBase::SaveState(const char *opt)
 		s=q;	q=strchr(s,';');
 		if(q)	{	*q=0;	q++;	}
 		mgl_strtrim(s);		char *a=s;
-		long n=mglFindArg(s);	if(n>0)	{	s[n]=0;		s=s+n+1;	}
+		long n=mglFindArg(s);	if(n>0)	{	s[n]=0;	s=s+n+1;	}
 		mgl_strtrim(a);		char *b=s;
 		n=mglFindArg(s);	if(n>0)	{	s[n]=0;		s=s+n+1;	}
 		mgl_strtrim(b);
@@ -1447,7 +1575,7 @@ mreal mglBase::SaveState(const char *opt)
 			if(a[0]=='x')		{	Min.x=ff;	Max.x=ss;	}
 			else if(a[0]=='y')	{	Min.y=ff;	Max.y=ss;	}
 			else if(a[0]=='z')	{	Min.z=ff;	Max.z=ss;	}
-//			else if(a[0]=='c')	{	Min.c=ff;	Max.c=ss;	}	// Bad idea since there is formula for coloring
+// NOTE!	else if(a[0]=='c')	{	Min.c=ff;	Max.c=ss;	}	// Bad idea since there is formula for coloring
 		}
 		else if(!strcmp(a,"cut"))		SetCut(ff!=0);
 		else if(!strcmp(a,"meshnum"))	SetMeshNum(ff);
