@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -77,17 +77,23 @@ int MGL_EXPORT_PURE mgl_get_warn(HMGL gr)	{	return gr->GetWarn();	}
 void MGL_EXPORT mgl_set_warn(HMGL gr, int code, const char *txt)
 {	gr->SetWarn(code,txt);	}
 extern bool mglPrintWarn;
+extern MGL_EXPORT std::string *mglGlobalMess;	///< Buffer for receiving global messages
 void MGL_EXPORT mgl_set_global_warn(const char *txt)
 {
 	if(txt && *txt)
 	{
-		mglGlobalMess += txt;	mglGlobalMess += '\n';
+		*mglGlobalMess += txt;	*mglGlobalMess += '\n';
 		if(mglPrintWarn)	fprintf(stderr,_("Global message - %s\n"),txt);
 	}
 }
 void MGL_EXPORT mgl_set_global_warn_(const char *txt, int l)
 {	char *s=new char[l+1];	memcpy(s,txt,l);	s[l]=0;	mgl_set_global_warn(s);	delete []s;	}
-MGL_EXPORT_PURE const char *mgl_get_global_warn()	{	return mglGlobalMess.c_str();	}
+void MGL_EXPORT mgl_clear_global_warn()
+{	*mglGlobalMess = "";	}
+void MGL_EXPORT mgl_clear_global_warn_()
+{	mgl_clear_global_warn();	}
+MGL_EXPORT_PURE const char *mgl_get_global_warn()
+{	return mglGlobalMess->empty()?"":mglGlobalMess->c_str();	}
 int MGL_EXPORT mgl_get_global_warn_(char *out, int len)
 {
 	const char *res = mgl_get_global_warn();
@@ -206,6 +212,7 @@ void MGL_EXPORT mgl_set_scale_text(HMGL gr, int enable)		{	gr->set(!enable, MGL_
 void MGL_EXPORT mgl_set_mark_size(HMGL gr, double size)		{	gr->SetMarkSize(size);	}
 void MGL_EXPORT mgl_set_arrow_size(HMGL gr, double size)	{	gr->SetArrowSize(size);	}
 void MGL_EXPORT mgl_set_font_size(HMGL gr, double size)		{	gr->SetFontSize(size);	}
+void MGL_EXPORT mgl_set_font_hscale(HMGL gr, double scale)	{	gr->SetFontHscale(scale);	}
 void MGL_EXPORT mgl_set_font_def(HMGL gr, const char *fnt)	{	gr->SetFontDef(fnt);	}
 void MGL_EXPORT mgl_load_font(HMGL gr, const char *name, const char *path)
 {	gr->LoadFont(name,path);	}
@@ -219,6 +226,7 @@ void MGL_EXPORT mgl_set_scale_text_(uintptr_t *gr, int *enable)	{	mgl_set_scale_
 void MGL_EXPORT mgl_set_mark_size_(uintptr_t *gr, mreal *size)		{	_GR_->SetMarkSize(*size);	}
 void MGL_EXPORT mgl_set_arrow_size_(uintptr_t *gr, mreal *size)	{	_GR_->SetArrowSize(*size);	}
 void MGL_EXPORT mgl_set_font_size_(uintptr_t *gr, mreal *size)		{	_GR_->SetFontSize(*size);	}
+void MGL_EXPORT mgl_set_font_hscale_(uintptr_t *gr, double *scale)	{	_GR_->SetFontHscale(*scale);	}
 void MGL_EXPORT mgl_set_font_def_(uintptr_t *gr, const char *name, int l)
 {	char *s=new char[l+1];		memcpy(s,name,l);	s[l]=0;
 	_GR_->SetFontDef(s);	delete []s;	}
@@ -232,13 +240,13 @@ void MGL_EXPORT mgl_restore_font_(uintptr_t *gr)	{	_GR_->RestoreFont();	}
 void MGL_EXPORT mgl_define_symbol_(uintptr_t *gr, char *id, uintptr_t *x, uintptr_t *y, int)
 {	_GR_->DefineGlyph(_DA_(x),_DA_(y),id?*id:0);	}
 //-----------------------------------------------------------------------------
-extern mglFont mglDefFont;
+extern mglFont *mglDefFont;
 void MGL_EXPORT mgl_def_font(const char *name, const char *path)
-{	mglDefFont.Load(name,path);	}
+{	mglDefFont->Load(name,path);	}
 void MGL_EXPORT mgl_def_font_(const char *name, const char *path,int l,int n)
 {	char *s=new char[l+1];		memcpy(s,name,l);	s[l]=0;
 	char *d=new char[n+1];		memcpy(d,path,n);	d[n]=0;
-	mglDefFont.Load(name,path);	delete []s;		delete []d;	}
+	mglDefFont->Load(name,path);	delete []s;		delete []d;	}
 //-----------------------------------------------------------------------------
 int MGL_EXPORT mgl_check_version(const char *ver)
 {	double v=0;	int r = sscanf(ver,"2.%lg",&v);
@@ -247,6 +255,11 @@ int MGL_EXPORT mgl_check_version_(const char *ver, int l)
 {	char *s=new char[l+1];		memcpy(s,ver,l);	s[l]=0;
 	int r=mgl_check_version(s);	delete []s;	return r;	}
 //-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_set_tex_parse(HMGL gr, int val)
+{	gr->GetFont()->parse=val;	}
+void MGL_EXPORT mgl_set_tex_parse_(uintptr_t *gr, int *val)
+{	_GR_->GetFont()->parse=*val;	}
+//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_start_group(HMGL gr, const char *s)	{	gr->StartAutoGroup(s);	}
 void MGL_EXPORT mgl_end_group(HMGL gr)	{	gr->EndGroup();	}
 void MGL_EXPORT mgl_start_group_(uintptr_t *gr, const char *name,int l)
