@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -21,8 +21,6 @@
 #include "mgl2/font.h"
 #include "mgl2/canvas.h"
 //-----------------------------------------------------------------------------
-MGL_EXPORT std::string mglGlobalMess;	///< Buffer for receiving global messages
-//-----------------------------------------------------------------------------
 mglCanvas::mglCanvas(int w, int h) : mglBase()
 {
 	clr(MGL_DISABLE_SCALE);
@@ -228,12 +226,12 @@ GifFileType *gif;*/
 	ax.Clear();	ay.Clear();	az.Clear();	ac.Clear();
 	mgl_clear_fft();		DefMaskAn=0;	ResetMask();
 	SetTickRotate(true);	SetTickSkip(true);
-	SetWarn(mglWarnNone,"");	mglGlobalMess = "";
+	SetWarn(mglWarnNone,"");	mgl_clear_global_warn();
 	ObjId = -1;	HighId = INT_MIN;
 	SetFunc(0,0);	CutOff(0);	Ternary(0);
 	Stop=false;	event_cb = NULL;	event_par=NULL;
 	SetRanges(mglPoint(-1,-1,-1,-1), mglPoint(1,1,1,1));
-	SetOrigin(NAN,NAN,NAN,NAN);
+	SetOrigin(NAN,NAN,NAN,NAN);	set(0, MGL_NO_ORIGIN);
 	SetBarWidth(0.7);	SetMarkSize(1);	SetArrowSize(1);
 	SetAlphaDef(0.5);		FontDef[0]=0;
 	SetTranspType(0);		SetMeshNum(0);	// NOTE: default MeshNum=0
@@ -965,7 +963,7 @@ void mglCanvas::Legend(const std::vector<mglText> &leg, mreal x, mreal y, const
 	strcpy(ff,fmt?fmt:"");	strcat(ff,":L");	Push();
 	if((pA=strchr(ff,'A')))
 	{	*pA = ' ';	InPlot(0,1,0,1,false);	iw=B1.b[0];	ih=B1.b[4];	}
-	else if(mglchr(font,'A'))
+	else if(strchr(font,'A'))
 	{	InPlot(0,1,0,1,false);	iw=B1.b[0];	ih=B1.b[4];	}
 	else	{	iw=B1.b[0]/B1.pf;	ih=B1.b[4]/B1.pf;	}
 	// find sizes
@@ -986,7 +984,7 @@ void mglCanvas::Legend(const std::vector<mglText> &leg, mreal x, mreal y, const
 		nrow = 1+(n-1)/j;
 		ncol = (n+nrow-1)/nrow;
 	}
-	if(mglchr(font,'^'))	// use "external" positioning
+	if(strchr(font,'^'))	// use "external" positioning
 	{
 		x = x>=0.5 ? x*iw : x*iw-w*ncol-2*dx;
 		y = y>=0.5 ? y*ih : y*ih-h*nrow-2*dy;
@@ -1001,6 +999,7 @@ void mglCanvas::Legend(const std::vector<mglText> &leg, mreal x, mreal y, const
 	mglPoint p,q(NAN,NAN,NAN);
 
 	mreal cc = AddTexture(font);
+	SetMask("");
 	mreal c1,c2;	//=AddTexture(char(k1?k1:'w')), c2=AddTexture(char(k2?k2:'k'));
 	if(cc<2 || Txt[long(cc+0.5)].n==0)
 	{	c1 = AddTexture('w');	cc = c2 = AddTexture('k');	}
@@ -1189,7 +1188,7 @@ void mglCanvas::StartAutoGroup (const char *lbl)
 //-----------------------------------------------------------------------------
 void mglCanvas::EndGroup()
 {
-	LoadState();
+	LoadState();	SetMask("");
 	if(Quality&MGL_DRAW_LMEM)
 	{
 		Pnt.clear();	Prm.clear();	Ptx.clear();	ClearPrmInd();
