@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -21,10 +21,44 @@
 #include "mgl2/canvas_cf.h"
 #include "mgl2/eval.h"
 #include "mgl2/evalc.h"
+#include "mgl2/font.h"
 //-----------------------------------------------------------------------------
 #undef _GR_
 #define _GR_	((mglCanvas *)(*gr))
 //-----------------------------------------------------------------------------
+MGL_EXPORT std::string *mglGlobalMess;	///< Buffer for receiving global messages
+MGL_EXPORT mglCanvas *mglDefaultGr=NULL;	///< Default HMGL object
+extern mglFont *mglDefFont;
+#if MGL_HAVE_PTHREAD
+pthread_mutex_t mutexRnd;
+#endif
+//-----------------------------------------------------------------------------
+float MGL_EXPORT mgl_cos[360];
+void MGL_EXPORT MGL_FUNC_INIT mgl_init()	// TODO try to add ld option: "-init mgl_init"
+{
+	static bool ini=true;
+	if(ini)
+	{
+		ini = false;
+		mglGlobalMess = new std::string;
+		mgl_textdomain(NULL,"");
+#if MGL_HAVE_PTHREAD
+		pthread_mutex_init(&mutexRnd,0);
+#endif
+#ifndef WIN32	// win32 don't initialized threads before main()
+#pragma omp parallel for
+#endif
+		for(long i=0;i<360;i++)	mgl_cos[i] = cos(i*M_PI/180.);
+		if(!mglDefFont)		mglDefFont = new mglFont(MGL_DEF_FONT_NAME);
+		if(!mglDefaultGr)	mglDefaultGr = new mglCanvas(600,400);
+	}
+}
+void MGL_EXPORT MGL_FUNC_FINI mgl_fini()	// TODO try to add ld option: "-fini mgl_fini"
+{
+	if(mglDefaultGr)	{	delete mglDefaultGr;mglDefaultGr=NULL;	}
+	if(mglDefFont)		{	delete mglDefFont;	mglDefFont=NULL;	}
+}
+//-----------------------------------------------------------------------------
 MGL_EXPORT const unsigned char *mgl_get_rgb(HMGL gr)
 {	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	return g?g->GetBits():0;	}
 MGL_EXPORT const unsigned char *mgl_get_rgba(HMGL gr)
@@ -111,6 +145,10 @@ void MGL_EXPORT mgl_clf_str(HMGL gr, const char *col)
 {	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	if(g)	g->Clf(col);	}
 void MGL_EXPORT mgl_load_background(HMGL gr, const char *fn, double alpha)
 {	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	if(g)	g->LoadBackground(fn,alpha);	}
+void MGL_EXPORT mgl_load_background_ext(HMGL gr, const char *fn, const char *how, double alpha)
+{	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	if(g)	g->LoadBackground(fn,how,alpha);	}
+void MGL_EXPORT mgl_fill_background(HMGL gr, double r, double g, double b)
+{	mglCanvas *gg = dynamic_cast<mglCanvas *>(gr);	if(gg)	gg->FillBackground(mglColor(r,g,b));	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_subplot_d(HMGL gr, int nx,int ny,int m,const char *style,double dx,double dy)
 {
@@ -226,6 +264,12 @@ void MGL_EXPORT mgl_clf_str_(uintptr_t *gr, const char *col, int l)
 void MGL_EXPORT mgl_load_background_(uintptr_t *gr, const char *fn, mreal *a, int l)
 {	char *s=new char[l+1];	memcpy(s,fn,l);	s[l]=0;
 	mgl_load_background(_GR_,s,*a);	delete []s;	}
+void MGL_EXPORT mgl_load_background_ext_(uintptr_t *gr, const char *fn, const char *how, mreal *a, int l,int m)
+{	char *s=new char[l+1];	memcpy(s,fn,l);		s[l]=0;
+	char *h=new char[m+1];	memcpy(h,how,l);	h[m]=0;
+	mgl_load_background_ext(_GR_,s,h,*a);	delete []s;	delete []h;	}
+void MGL_EXPORT mgl_fill_background_(uintptr_t *gr, double *r, double *g, double *b)
+{	mgl_fill_background(_GR_,*r,*g,*b);	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_subplot_d_(uintptr_t *gr, int *nx,int *ny,int *m,const char *st, mreal *dx, mreal *dy,int l)
 {	char *s=new char[l+1];	memcpy(s,st,l);	s[l]=0;
@@ -290,6 +334,12 @@ double mgl_size_scl = 1;
 HMGL MGL_EXPORT mgl_create_graph(int width, int height)
 {	return new mglCanvas(width,height);	}
 void MGL_EXPORT mgl_delete_graph(HMGL gr)	{	if(gr)	delete gr;	}
+HMGL MGL_EXPORT mgl_default_graph()
+{
+	if(!mglDefaultGr)
+		mglDefaultGr = new mglCanvas(600,400);
+	return mglDefaultGr;
+}
 void MGL_EXPORT mgl_set_size_scl(double scl){	if(scl>0)	mgl_size_scl = scl;	}
 void MGL_EXPORT mgl_set_size(HMGL gr, int width, int height)
 {
@@ -384,6 +434,7 @@ void MGL_EXPORT mgl_set_legend_marks(HMGL gr, int num)
 uintptr_t MGL_EXPORT mgl_create_graph_(int *width, int *height)
 {	return uintptr_t(new mglCanvas(*width,*height));	}
 void MGL_EXPORT mgl_delete_graph_(uintptr_t *gr)	{	delete _GR_;	}
+uintptr_t MGL_EXPORT mgl_default_graph_()	{	return uintptr_t(&mglDefaultGr);	}
 void MGL_EXPORT mgl_set_size_scl_(double *scl)	{	mgl_set_size_scl(*scl);	}
 void MGL_EXPORT mgl_set_size_(uintptr_t *gr, int *width, int *height)
 {	mgl_set_size(_GR_,*width,*height);	}
@@ -451,13 +502,13 @@ void MGL_EXPORT mgl_label_(uintptr_t *gr, const char *dir, const char *text, mre
 	char *o=new char[m+1];	memcpy(o,opt,m);	o[m]=0;
 	_GR_->Label(*dir, s, *pos, o);	delete []s;	delete []o;	}
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_colorbar_(uintptr_t *gr, const char *sch,int l)
+void MGL_EXPORT mgl_colorbar_(uintptr_t *gr, const char *sch, int l)
 {	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	_GR_->Colorbar(s);	delete []s;	}
 void MGL_EXPORT mgl_colorbar_ext_(uintptr_t *gr, const char *sch, mreal *x, mreal *y, mreal *w, mreal *h, int l)
 {	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	_GR_->Colorbar(s,*x,*y,*w,*h);	delete []s;	}
-void MGL_EXPORT mgl_colorbar_val_(uintptr_t *gr, uintptr_t *dat, const char *sch,int l)
+void MGL_EXPORT mgl_colorbar_val_(uintptr_t *gr, uintptr_t *dat, const char *sch, int l)
 {	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
 	_GR_->Colorbar(_DA_(dat), s);	delete []s;	}
 void MGL_EXPORT mgl_colorbar_val_ext_(uintptr_t *gr, uintptr_t *dat, const char *sch, mreal *x, mreal *y, mreal *w, mreal *h, int l)
