@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -1114,20 +1114,20 @@ void MGL_EXPORT mgl_datac_diffr(HADT d, const char *how, mreal q)
 		p[0]=nz;	p[1]=nx*ny;	p[2]=0;
 		mglStartThreadC(mgl_difr,0,nx*ny,d->a,&qq,0,p);
 	}
-	if(mglchr(how,'y') && ny>1 && !axial)
+	if(mglchr(how,'y') && ny>1)
 	{
 		p[0]=ny;	p[1]=nx;	p[2]=0;
 		mglStartThreadC(mgl_difr,0,nx*nz,d->a,&qq,0,p);
 	}
-	if(mglchr(how,'x') && nx>1 && !axial)
-	{
-		p[0]=nx;	p[1]=1;	p[2]=0;
-		mglStartThreadC(mgl_difr,0,ny*nz,d->a,&qq,0,p);
-	}
 	if(axial && nx>1)
 	{
 		p[0]=nx;	p[1]=1;	p[2]=1;
-		mglStartThreadC(mgl_difr,0,ny*nz,0,&qq,0,p);
+		mglStartThreadC(mgl_difr,0,ny*nz,d->a,&qq,0,p);
+	}
+	else if(mglchr(how,'x') && nx>1)
+	{
+		p[0]=nx;	p[1]=1;	p[2]=0;
+		mglStartThreadC(mgl_difr,0,ny*nz,d->a,&qq,0,p);
 	}
 }
 //-----------------------------------------------------------------------------
