@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -34,7 +34,6 @@
 
 inline bool isn(char ch)	{return ch=='\n';}
 MGL_NO_EXPORT char *mgl_read_gz(gzFile fp);
-HADT MGL_NO_EXPORT mglFormulaCalcC(const char *str, const std::vector<mglDataA*> &head);
 //-----------------------------------------------------------------------------
 HADT MGL_EXPORT mgl_create_datac()	{	return new mglDataC;	}
 HADT MGL_EXPORT mgl_create_datac_size(long nx, long ny, long nz){	return new mglDataC(nx,ny,nz);	}
@@ -42,14 +41,14 @@ HADT MGL_EXPORT mgl_create_datac_file(const char *fname)		{	return new mglDataC(
 void MGL_EXPORT mgl_delete_datac(HADT d)	{	if(d)	delete d;	}
 //-----------------------------------------------------------------------------
 uintptr_t MGL_EXPORT mgl_create_datac_()
-{	return uintptr_t(new mglDataC());	}
+{	return uintptr_t(mgl_create_datac());	}
 uintptr_t MGL_EXPORT mgl_create_datac_size_(int *nx, int *ny, int *nz)
-{	return uintptr_t(new mglDataC(*nx,*ny,*nz));	}
+{	return uintptr_t(mgl_create_datac_size(*nx,*ny,*nz));	}
 uintptr_t MGL_EXPORT mgl_create_datac_file_(const char *fname,int l)
 {	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
-	uintptr_t r = uintptr_t(new mglDataC(s));	delete []s;	return r;	}
+	uintptr_t r = uintptr_t(mgl_create_datac_file(s));	delete []s;	return r;	}
 void MGL_EXPORT mgl_delete_datac_(uintptr_t *d)
-{	if(_DC_)	delete _DC_;	}
+{	mgl_delete_datac(_DC_);	}
 //-----------------------------------------------------------------------------
 cmdual MGL_EXPORT mgl_atoc(const char *s, int adv)
 {
@@ -882,8 +881,9 @@ int MGL_EXPORT mgl_datac_read_hdf(HADT d,const char *fname,const char *data)
 	hf = H5Fopen(fname, H5F_ACC_RDONLY, H5P_DEFAULT);
 	if(hf<0)	return 0;
 	hd = H5Dopen(hf,data);
-	if(hd<0)	return 0;
+	if(hd<0)	{	H5Fclose(hf);	return 0;	}
 	hs = H5Dget_space(hd);
+	if(hs<0)	{	H5Dclose(hd);	H5Fclose(hf);	return 0;	}
 	rank = H5Sget_simple_extent_ndims(hs);
 	if(rank>0 && rank<=4)
 	{
