@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -40,7 +40,6 @@
 #endif
 
 inline bool isn(char ch)	{return ch=='\n';}
-HMDT MGL_NO_EXPORT mglFormulaCalc(const char *str, const std::vector<mglDataA*> &head);
 //-----------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_create_data()	{	return new mglData;	}
 HMDT MGL_EXPORT mgl_create_data_size(long nx, long ny, long nz){	return new mglData(nx,ny,nz);	}
@@ -48,14 +47,14 @@ HMDT MGL_EXPORT mgl_create_data_file(const char *fname)		{	return new mglData(fn
 void MGL_EXPORT mgl_delete_data(HMDT d)	{	if(d)	delete d;	}
 //-----------------------------------------------------------------------------
 uintptr_t MGL_EXPORT mgl_create_data_()
-{	return uintptr_t(new mglData());	}
+{	return uintptr_t(mgl_create_data());	}
 uintptr_t MGL_EXPORT mgl_create_data_size_(int *nx, int *ny, int *nz)
-{	return uintptr_t(new mglData(*nx,*ny,*nz));	}
+{	return uintptr_t(mgl_create_data_size(*nx,*ny,*nz));	}
 uintptr_t MGL_EXPORT mgl_create_data_file_(const char *fname,int l)
 {	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
-	uintptr_t r = uintptr_t(new mglData(s));	delete []s;	return r;	}
+	uintptr_t r = uintptr_t(mgl_create_data_file(s));	delete []s;	return r;	}
 void MGL_EXPORT mgl_delete_data_(uintptr_t *d)
-{	if(_DT_)	delete _DT_;	}
+{	mgl_delete_data(_DT_);	}
 //-----------------------------------------------------------------------------
 void mglFromStr(HMDT d,char *buf,long NX,long NY,long NZ)
 {
@@ -309,6 +308,33 @@ std::string MGL_EXPORT mgl_str_arg(const std::string &str, char ch, int n1, int
 	return res;
 }
 //-----------------------------------------------------------------------------
+/// Get section separated by symbol ch. This is analog of QString::section().
+std::vector<std::wstring> MGL_EXPORT mgl_wcs_args(const std::wstring &str, wchar_t ch)
+{
+	std::vector<size_t> pos;	pos.push_back(0);
+	for(size_t p=0; p != std::string::npos;)
+	{	p=str.find(ch,p+1);	pos.push_back(p?p+1:0);	}
+	std::vector<std::wstring> res;
+	for(size_t i=0;i<pos.size()-1;i++)
+		res.push_back(str.substr(pos[i],pos[i+1]-pos[i]-1));
+	return res;
+}
+//-----------------------------------------------------------------------------
+/// Get section separated by symbol ch. This is analog of QString::section().
+std::wstring MGL_EXPORT mgl_wcs_arg(const std::wstring &str, wchar_t ch, int n1, int n2)
+{
+	std::vector<size_t> pos;	pos.push_back(0);
+	for(size_t p=0; p != std::string::npos;)
+	{	p=str.find(ch,p+1);	pos.push_back(p?p+1:0);	}
+	std::wstring res;
+	if(n2<0)	n2=n1;
+	if(n1<0 || n1>=long(pos.size())-1 || n2<n1)	return res;
+	if(n2>=long(pos.size()))	n2=pos.size()-1;
+	res = str.substr(pos[n1],pos[n2+1]-pos[n1]-1);
+	if(res.size()==1 && res[0]==ch)	res.clear();
+	return res;
+}
+//-----------------------------------------------------------------------------
 /// Get string from number.
 std::string MGL_EXPORT mgl_str_num(double val)
 {	char buf[32];	snprintf(buf,32,"%g",val);	return std::string(buf);	}
@@ -374,6 +400,92 @@ MGL_NO_EXPORT char *mgl_read_gz(gzFile fp)
 	return buf;
 }
 //-----------------------------------------------------------------------------
+int MGL_EXPORT mgl_data_read_bin(HMDT dat, const char *fname, int kind)
+{
+	FILE *fp = fopen(fname,"rb");
+	if(!fp)	return 0;
+	fseek(fp,0,SEEK_END);
+	long len=ftell(fp), n=0;	// file length, and number of elements
+	fseek(fp,0,SEEK_SET);
+	switch(kind)
+	{
+		case 0:	// double type
+			n = len/sizeof(double);
+			if(n>0)
+			{
+				double *a = new double[n];
+				n = fread(a,sizeof(double),n,fp);
+				dat->Create(n);
+				for(long i=0;i<n;i++)	dat->a[i]=a[i];
+			}
+			break;
+		case 1:	// float type
+			n = len/sizeof(float);
+			if(n>0)
+			{
+				float *a = new float[n];
+				n = fread(a,sizeof(float),n,fp);
+				dat->Create(n);
+				for(long i=0;i<n;i++)	dat->a[i]=a[i];
+			}
+			break;
+		case 2:	// long double type
+			n = len/sizeof(long double);
+			if(n>0)
+			{
+				long double *a = new long double[n];
+				n = fread(a,sizeof(long double),n,fp);
+				dat->Create(n);
+				for(long i=0;i<n;i++)	dat->a[i]=a[i];
+			}
+			break;
+		case 3:	// long int type
+			n = len/sizeof(long);
+			if(n>0)
+			{
+				long *a = new long[n];
+				n = fread(a,sizeof(long),n,fp);
+				dat->Create(n);
+				for(long i=0;i<n;i++)	dat->a[i]=a[i];
+			}
+			break;
+		case 4:	// int type
+			n = len/sizeof(int);
+			if(n>0)
+			{
+				int *a = new int[n];
+				n = fread(a,sizeof(int),n,fp);
+				dat->Create(n);
+				for(long i=0;i<n;i++)	dat->a[i]=a[i];
+			}
+			break;
+		case 5:	// short int type
+			n = len/sizeof(short);
+			if(n>0)
+			{
+				short *a = new short[n];
+				n = fread(a,sizeof(short),n,fp);
+				dat->Create(n);
+				for(long i=0;i<n;i++)	dat->a[i]=a[i];
+			}
+			break;
+		case 6:	// char type
+			n = len/sizeof(char);
+			if(n>0)
+			{
+				char *a = new char[n];
+				n = fread(a,sizeof(char),n,fp);
+				dat->Create(n);
+				for(long i=0;i<n;i++)	dat->a[i]=a[i];
+			}
+			break;
+	}
+	return 1;
+}
+int MGL_EXPORT mgl_data_read_bin_(uintptr_t *d, const char *fname,int *kind,int l)
+{	char *s=new char[l+1];		memcpy(s,fname,l);	s[l]=0;
+	int r = mgl_data_read_bin(_DT_, s, *kind);	delete []s;		return r;	}
+//-----------------------------------------------------------------------------
 int MGL_EXPORT mgl_data_read(HMDT d, const char *fname)
 {
 	long l=1,m=1,k=1,i;
@@ -1010,7 +1122,7 @@ static void *mgl_modify(void *par)
 void MGL_EXPORT mgl_data_modify(HMDT d, const char *eq,long dim)
 {
 	long nx=d->nx, ny=d->ny, nz=d->nz, par[3]={nx,ny,nz};
-	if(dim<=0)	mgl_data_modify_vw(d,eq,0,0);	// fastes variant for whole array
+	if(dim<=0)	mgl_data_modify_vw(d,eq,0,0);	// fastest variant for whole array
 	else if(nz>1)	// 3D array
 	{
 		mglFormula f(eq);
@@ -1093,6 +1205,61 @@ int MGL_EXPORT mgl_data_read_hdf4(HMDT ,const char *,const char *)
 #endif
 //-----------------------------------------------------------------------------
 #if MGL_HAVE_HDF5
+void MGL_EXPORT mgl_dual_save_hdf(mdual val,const char *fname,const char *data,int rewrite)
+{
+	hid_t hf,hd,hs;
+	hsize_t dims[4]={1,2};
+	long rank = 2, res;
+	H5Eset_auto(0,0);
+	res=H5Fis_hdf5(fname);
+	if(res>0 && !rewrite)	hf = H5Fopen(fname, H5F_ACC_RDWR, H5P_DEFAULT);
+	else	hf = H5Fcreate(fname, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
+	if(hf<0)	return;
+	hs = H5Screate_simple(rank, dims, 0);
+#if MGL_USE_DOUBLE
+	hid_t mem_type_id = H5T_NATIVE_DOUBLE;
+#else
+	hid_t mem_type_id = H5T_NATIVE_FLOAT;
+#endif
+	hd = H5Dcreate(hf, data, mem_type_id, hs, H5P_DEFAULT);
+	H5Dwrite(hd, mem_type_id, hs, hs, H5P_DEFAULT, &val);
+	H5Dclose(hd);	H5Sclose(hs);	H5Fclose(hf);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_real_save_hdf(double val,const char *fname,const char *data,int rewrite)
+{
+	hid_t hf,hd,hs;
+	hsize_t dims[3]={1,1,1};
+	long rank = 1, res;
+	H5Eset_auto(0,0);
+	res=H5Fis_hdf5(fname);
+	if(res>0 && !rewrite)	hf = H5Fopen(fname, H5F_ACC_RDWR, H5P_DEFAULT);
+	else	hf = H5Fcreate(fname, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
+	if(hf<0)	return;
+	hs = H5Screate_simple(rank, dims, 0);
+	hid_t mem_type_id = H5T_NATIVE_DOUBLE;
+	hd = H5Dcreate(hf, data, mem_type_id, hs, H5P_DEFAULT);
+	H5Dwrite(hd, mem_type_id, hs, hs, H5P_DEFAULT, &val);
+	H5Dclose(hd);	H5Sclose(hs);	H5Fclose(hf);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_int_save_hdf(long val,const char *fname,const char *data,int rewrite)
+{
+	hid_t hf,hd,hs;
+	hsize_t dims[3]={1,1,1};
+	long rank = 1, res;
+	H5Eset_auto(0,0);
+	res=H5Fis_hdf5(fname);
+	if(res>0 && !rewrite)	hf = H5Fopen(fname, H5F_ACC_RDWR, H5P_DEFAULT);
+	else	hf = H5Fcreate(fname, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
+	if(hf<0)	return;
+	hs = H5Screate_simple(rank, dims, 0);
+	hid_t mem_type_id = H5T_NATIVE_LONG;
+	hd = H5Dcreate(hf, data, mem_type_id, hs, H5P_DEFAULT);
+	H5Dwrite(hd, mem_type_id, hs, hs, H5P_DEFAULT, &val);
+	H5Dclose(hd);	H5Sclose(hs);	H5Fclose(hf);
+}
+//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_data_save_hdf(HCDT dat,const char *fname,const char *data,int rewrite)
 {
 	const mglData *d = dynamic_cast<const mglData *>(dat);	// NOTE: slow for non-mglData
@@ -1127,8 +1294,9 @@ int MGL_EXPORT mgl_data_read_hdf(HMDT d,const char *fname,const char *data)
 	hf = H5Fopen(fname, H5F_ACC_RDONLY, H5P_DEFAULT);
 	if(hf<0)	return 0;
 	hd = H5Dopen(hf,data);
-	if(hd<0)	return 0;
+	if(hd<0)	{	H5Fclose(hf);	return 0;	}
 	hs = H5Dget_space(hd);
+	if(hs<0)	{	H5Dclose(hd);	H5Fclose(hf);	return 0;	}
 	rank = H5Sget_simple_extent_ndims(hs);
 	if(rank>0 && rank<=3)
 	{
@@ -1195,6 +1363,12 @@ long MGL_EXPORT mgl_datas_hdf(const char *, char *, long )
 {	mgl_set_global_warn(_("HDF5 support was disabled. Please, enable it and rebuild MathGL."));	return 0;}
 int MGL_EXPORT mgl_data_read_hdf(HMDT ,const char *,const char *)
 {	mgl_set_global_warn(_("HDF5 support was disabled. Please, enable it and rebuild MathGL."));	return 0;}
+void MGL_EXPORT mgl_dual_save_hdf(mdual val,const char *fname,const char *data,int rewrite)
+{	mgl_set_global_warn(_("HDF5 support was disabled. Please, enable it and rebuild MathGL."));	}
+void MGL_EXPORT mgl_real_save_hdf(double val,const char *fname,const char *data,int rewrite)
+{	mgl_set_global_warn(_("HDF5 support was disabled. Please, enable it and rebuild MathGL."));	}
+void MGL_EXPORT mgl_int_save_hdf(long val,const char *fname,const char *data,int rewrite)
+{	mgl_set_global_warn(_("HDF5 support was disabled. Please, enable it and rebuild MathGL."));	}
 #endif
 //-----------------------------------------------------------------------------
 int MGL_EXPORT mgl_data_read_hdf_(uintptr_t *d, const char *fname, const char *data,int l,int n)
@@ -1208,6 +1382,14 @@ void MGL_EXPORT mgl_data_save_hdf_(uintptr_t *d, const char *fname, const char *
 long MGL_EXPORT mgl_datas_hdf_(const char *fname, char *buf, int l, int size)
 {	char *s=new char[l+1];		memcpy(s,fname,l);	s[l]=0;
 	int r = mgl_datas_hdf(s,buf,size);	delete []s;	return r;	}
+void MGL_EXPORT mgl_real_save_hdf_(double *val,const char *fname,const char *data,int *rewrite,int l,int n)
+{	char *s=new char[l+1];		memcpy(s,fname,l);	s[l]=0;
+	char *t=new char[n+1];		memcpy(t,data,n);	t[n]=0;
+	mgl_real_save_hdf(*val,s,t,*rewrite);	delete []s;	delete []t;	}
+void MGL_EXPORT mgl_int_save_hdf_(long *val,const char *fname,const char *data,int *rewrite,int l,int n)
+{	char *s=new char[l+1];		memcpy(s,fname,l);	s[l]=0;
+	char *t=new char[n+1];		memcpy(t,data,n);	t[n]=0;
+	mgl_int_save_hdf(*val,s,t,*rewrite);	delete []s;	delete []t;	}
 //-----------------------------------------------------------------------------
 bool MGL_EXPORT mgl_add_file(long &kx,long &ky, long &kz, mreal *&b, mglData *d,bool as_slice)
 {
