@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -71,7 +71,7 @@ EQ_LAST		// id of last entry
 //-----------------------------------------------------------------------------
 int mglFormulaC::Error=0;
 bool MGL_LOCAL_PURE mglCheck(char *str,int n);
-int MGL_LOCAL_PURE mglFindInText(const char *str, const char *lst);
+long MGL_LOCAL_PURE mglFindInText(const char *str, const char *lst);
 //-----------------------------------------------------------------------------
 mglFormulaC::~mglFormulaC()
 {
@@ -81,13 +81,12 @@ mglFormulaC::~mglFormulaC()
 }
 //-----------------------------------------------------------------------------
 // Formula constructor (automatically parse and "compile" formula)
-mglFormulaC::mglFormulaC(const char *string)
+mglFormulaC::mglFormulaC(const char *string):Res(0)
 {
 	dat = tmp = NULL;
 	dx1=dy1=dz1=0;	dx2=dy2=dz2=1;
-	Error=0;
+	Error=Kod=0;
 	Left=Right=0;
-	Res=0; Kod=0;
 	if(!string)	{	Kod = EQ_NUM;	Res = 0;	return;	}
 	char *str = new char[strlen(string)+1];
 	strcpy(str,string);
@@ -129,7 +128,7 @@ mglFormulaC::mglFormulaC(const char *string)
 		delete []str;	return;
 	}
 	n=mglFindInText(str,"+-");				// normal priority -- additions
-	if(n>=0 && (n<2 || str[n-1]!='e' || (str[n-2]!='.' && !isdigit(str[n-2]))))
+	if(n>=0 && (n<2 || !strchr("eE",str[n-1]) || (str[n-2]!='.' && !isdigit(str[n-2]))))
 	{
 		if(str[n]=='+') Kod=EQ_ADD; else Kod=EQ_SUB;
 		str[n]=0;
