@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -707,6 +707,20 @@ int static mgls_join(mglGraph *, long , mglArg *a, const char *k, const char *)
 	return res;
 }
 //-----------------------------------------------------------------------------
+int static mgls_keep(mglGraph *, long , mglArg *a, const char *k, const char *)
+{
+	int res=0;
+	if(k[0]=='d' && a[0].d->temp)	return 5;
+	mglData *d = dynamic_cast<mglData *>(a[0].d);
+	mglDataC *c = dynamic_cast<mglDataC *>(a[0].d);
+	if(d && !strcmp(k,"dsn"))		d->Keep(a[1].s.s,a[2].v);
+	else if(c && !strcmp(k,"dsn"))	c->Keep(a[1].s.s,a[2].v);
+	else if(d && !strcmp(k,"dsnn"))	d->Keep(a[1].s.s,a[2].v,a[3].v);
+	else if(c && !strcmp(k,"dsnn"))	c->Keep(a[1].s.s,a[2].v,a[3].v);
+	else res = 1;
+	return res;
+}
+//-----------------------------------------------------------------------------
 int static mgls_limit(mglGraph *, long , mglArg *a, const char *k, const char *)
 {
 	int res=0;
@@ -719,6 +733,26 @@ int static mgls_limit(mglGraph *, long , mglArg *a, const char *k, const char *)
 	return res;
 }
 //-----------------------------------------------------------------------------
+int static mgls_first(mglGraph *, long , mglArg *a, const char *k, const char *)
+{
+	int res=0;
+	if(k[0]=='d' && a[0].d->temp)	return 5;
+	mglData *d = dynamic_cast<mglData *>(a[0].d);
+	if(d && !strcmp(k,"ddsn"))	*d = mglData(true,mgl_data_first_dir(a[1].d,a[2].s.s,a[3].v));
+	else res = 1;
+	return res;
+}
+//-----------------------------------------------------------------------------
+int static mgls_last(mglGraph *, long , mglArg *a, const char *k, const char *)
+{
+	int res=0;
+	if(k[0]=='d' && a[0].d->temp)	return 5;
+	mglData *d = dynamic_cast<mglData *>(a[0].d);
+	if(d && !strcmp(k,"ddsn"))	*d = mglData(true,mgl_data_last_dir(a[1].d,a[2].s.s,a[3].v));
+	else res = 1;
+	return res;
+}
+//-----------------------------------------------------------------------------
 int static mgls_max(mglGraph *, long , mglArg *a, const char *k, const char *)
 {
 	int res=0;
@@ -864,6 +898,10 @@ int static mgls_ode(mglGraph *, long , mglArg *a, const char *k, const char *)
 			*d = mglODE(a[1].s.s, a[2].s.s, *(a[3].d));
 		else if(!strcmp(k,"dssdnn"))
 			*d = mglODE(a[1].s.s, a[2].s.s, *(a[3].d), a[4].v, a[5].v);
+		if(!strcmp(k,"dsssd"))
+			*d = mglODEs(a[1].s.s, a[2].s.s, a[3].s.s[0], *(a[4].d));
+		else if(!strcmp(k,"dsssdnn"))
+			*d = mglODEs(a[1].s.s, a[2].s.s, a[3].s.s[0], *(a[4].d), a[5].v, a[6].v);
 	}
 	else if(c)
 	{
@@ -871,6 +909,10 @@ int static mgls_ode(mglGraph *, long , mglArg *a, const char *k, const char *)
 			*c = mglODEc(a[1].s.s, a[2].s.s, *(a[3].d));
 		else if(!strcmp(k,"dssdnn"))
 			*c = mglODEc(a[1].s.s, a[2].s.s, *(a[3].d), a[4].v, a[5].v);
+		if(!strcmp(k,"dsssd"))
+			*c = mglODEcs(a[1].s.s, a[2].s.s, a[3].s.s[0], *(a[4].d));
+		else if(!strcmp(k,"dsssdnn"))
+			*c = mglODEcs(a[1].s.s, a[2].s.s, a[3].s.s[0], *(a[4].d), a[5].v, a[6].v);
 	}
 	else res = 1;
 	return res;
@@ -1211,6 +1253,18 @@ int static mgls_readmat(mglGraph *gr, long , mglArg *a, const char *k, const cha
 	return res;
 }
 //-----------------------------------------------------------------------------
+int static mgls_readbin(mglGraph *gr, long , mglArg *a, const char *k, const char *)
+{
+	int res=0;
+	bool rr=true;
+	if(k[0]=='d' && a[0].d->temp)	return 5;
+	mglData *d = dynamic_cast<mglData *>(a[0].d);
+	if(d && !strcmp(k,"dsn"))		rr=d->ReadBin(a[1].s.s, mgl_int(a[2].v));
+	else res = 1;
+	if(!rr)	gr->SetWarn(mglWarnFile,"ReadBin");
+	return res;
+}
+//-----------------------------------------------------------------------------
 int static mgls_rearrange(mglGraph *, long , mglArg *a, const char *k, const char *)
 {
 	int res=0;
@@ -1317,6 +1371,12 @@ int static mgls_save(mglGraph *, long , mglArg *a, const char *k, const char *)
 {
 	int res=0;
 	if(!strcmp(k,"ds"))	a[0].d->Save(a[1].s.s);
+	else if(!strcmp(k,"ns"))
+	{
+		FILE *fp = fopen(a[1].s.s,"w");
+		fprintf(fp,"%g",double(a[0].v));
+		fclose(fp);
+	}
 	else if(!strcmp(k,"ss"))
 	{
 		FILE *fp = fopen(a[1].s.s,"a");
@@ -1344,6 +1404,8 @@ int static mgls_savehdf(mglGraph *, long , mglArg *a, const char *k, const char
 	int res=0;
 	if(!strcmp(k,"dss"))	a[0].d->SaveHDF(a[1].s.s, a[2].s.s);
 	else if(!strcmp(k,"dssn"))	a[0].d->SaveHDF(a[1].s.s, a[2].s.s,mgl_int(a[3].v));
+	else if(!strcmp(k,"nss"))	mgl_real_save_hdf(a[0].v, a[1].s.s, a[2].s.s, 0);
+	else if(!strcmp(k,"nssn"))	mgl_real_save_hdf(a[0].v, a[1].s.s, a[2].s.s, mgl_int(a[3].v));
 	else res = 1;
 	return res;
 }
@@ -1694,6 +1756,7 @@ mglCommand mgls_dat_cmd[] = {
 	{"minmax",_("Get positions of local maximums and minimums"),"minmax Res Dat", mgls_minmax ,4},
 	{"fill",_("Fill data linearly in range [v1, v2]"),"fill Var v1 v2 ['dir']|Var 'eq' [Vdat Wdat]", mgls_fill ,3},
 	{"fillsample",_("Fill x-,k-samples for transforms"),"fillsample Var 'how'", mgls_fillsample ,3},
+	{"first",_("Find first indexes of values larger val over direction"),"first Res Dat 'dir' val", mgls_first ,4},
 	{"fit",_("Fit data to formula"),"fit Res A 'eq' 'var' [Ini]|Res X A 'eq' 'var' [Ini]|Res X Y A 'eq' 'var' [Ini]|Res X Y Z A 'eq' 'var' [Ini]", mgls_fit ,4},
 	{"fits",_("Fit data to formula"),"fits Res A S 'eq' 'var' [Ini]|Res X A S 'eq' 'var' [Ini]|Res X Y A S 'eq' 'var' [Ini]|Res X Y Z A S 'eq' 'var' [Ini]", mgls_fits ,4},
 	{"fourier",_("In-place Fourier transform"),"fourier ReDat ImDat 'dir'|Cmplx 'dir'", mgls_fourier , 16},
@@ -1707,6 +1770,8 @@ mglCommand mgls_dat_cmd[] = {
 	{"integrate",_("Integrate data along direction(s)"),"integrate Dat 'dir'", mgls_integrate ,16},
 	{"jacobian",_("Get Jacobian"),"jacobian Res Xdat Ydat [Zdat]", mgls_jacobian ,4},
 	{"join",_("Join data arrays"),"join Dat Add1 ...", mgls_join ,3},
+	{"last",_("Find last indexes of values larger val over direction"),"last Res Dat 'dir' val", mgls_last ,4},
+	{"keep",_("Keep phase/amplitude along line"),"keep Dat 'dir' i [j]", mgls_keep ,16},
 	{"limit",_("Limit data to be inside [-v,v]"),"limit Dat v", mgls_limit ,16},
 	{"max",_("Find maximal value over direction"),"max Res Dat 'dir'", mgls_max ,4},
 	{"min",_("Find minimal value over direction"),"min Res Dat 'dir'", mgls_min ,4},
@@ -1717,7 +1782,7 @@ mglCommand mgls_dat_cmd[] = {
 	{"new",_("Create new data"),"new Dat nx ['eq']|Dat nx ny ['eq']|Dat nx ny nz ['eq']", mgls_new ,4},
 	{"norm",_("Normalize data"),"norm Dat v1 v2 [sym dim]", mgls_norm ,16},
 	{"normsl",_("Normalize data slice by slice"),"normsl Dat v1 v2 ['dir' keep sym] ", mgls_normsl ,16},
-	{"ode",_("Solve ODE"),"ode Res 'df' 'var' Ini [dt tmax]", mgls_ode ,4},
+	{"ode",_("Solve ODE"),"ode Res 'df' 'var' Ini [dt tmax]|Res 'df' 'var' 'brd' Ini [dt tmax]", mgls_ode ,4},
 	{"openhdf",_("Open all data arrays from HDF file"),"openhdf 'fname'", mgls_openhdf ,3},
 	{"pde",_("Solve PDE"),"pde Res 'ham' IniRe IniIm [dz k0]", mgls_pde ,4},
 	{"print",_("Immediately print the message"),"print 'message'|Dat|const", mgls_print ,3},
@@ -1730,6 +1795,7 @@ mglCommand mgls_dat_cmd[] = {
 	{"ray",_("Solve Hamiltonian ODE (find GO ray or trajectory)"),"ray Res 'ham' x0 y0 z0 px0 py0 pz0 [dt=0.1 tmax=10]", mgls_ray ,4},
 	{"read",_("Read data from file"),"read Dat 'file' [nx ny nz]|ReDat ImDat 'file' [nx ny nz]", mgls_read ,4},
 	{"readall",_("Read and join data from several files"),"readall Dat 'templ' [slice]|Dat 'templ' from to [step slice]", mgls_readall ,4},
+	{"readbin",_("Read data from binary file of specified type"),"readbin Dat 'file' type", mgls_readbin ,4},
 	{"readhdf",_("Read data with name 'id' from HDF file"),"readhdf Dat 'file' 'id'", mgls_readhdf ,4},
 	{"readmat",_("Read data from file with sizes specified in first row"),"readmat Dat 'file' [dim]", mgls_readmat ,4},
 	{"rearrange",_("Rearrange data dimensions"),"rearrange Dat mx [my mz]", mgls_rearrange ,3},
@@ -1738,8 +1804,8 @@ mglCommand mgls_dat_cmd[] = {
 	{"rkstep",_("Perform Runge-Kutta step"),"rkstep 'Diff1;Diff2;...' 'Var1;Var2;...' [dt]", mgls_rkstep, 6},
 	{"roll",_("Roll data along direction(s)"),"roll Dat 'dir' num", mgls_roll ,16},
 	{"roots",_("Find roots using data as initial values"), "roots Res 'func' Ini ['var']|Res 'func' ini ['var']|Res 'func' 'vars' Ini", mgls_roots ,4},
-	{"save",_("Save data to file"),"save Dat 'file'|'str' 'file' ['how']", mgls_save ,3},
-	{"savehdf",_("Save data to HDF5 file"),"savehdf Dat 'file' 'id' [rewrite]", mgls_savehdf ,3},
+	{"save",_("Save data to file"),"save Dat 'file'|val 'file'|'str' 'file' ['how']", mgls_save ,3},
+	{"savehdf",_("Save data to HDF5 file"),"savehdf Dat 'file' 'id' [rewrite]|val 'file' 'id' [rewrite]", mgls_savehdf ,3},
 	{"scanfile",_("Get formated data from file"),"scanfile Dat 'fname 'templ'", mgls_scanfile ,4},
 	{"section",_("Extract sub-array between values"),"section Res Dat id ['dir' val]|Res Dat Ids ['dir' val]", mgls_section ,4},
 	{"sew",_("Remove jump into the data, like phase jumps"),"sew Dat ['dir' da]", mgls_sew ,16},
