@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -65,7 +65,7 @@ int MGL_NO_EXPORT mgl_pnga_save(const char *fname, int w, int h, unsigned char *
 						PNG_FILTER_TYPE_DEFAULT);
 			png_set_rows(png_ptr, info_ptr, p);
 			png_write_png(png_ptr, info_ptr,  PNG_TRANSFORM_IDENTITY, 0);
-			png_write_end(png_ptr, info_ptr);
+//			png_write_end(png_ptr, info_ptr);
 		}
 		png_destroy_write_struct(&png_ptr, &info_ptr);
 	}
@@ -99,7 +99,7 @@ int MGL_NO_EXPORT mgl_png_save(const char *fname, int w, int h, unsigned char **
 						PNG_FILTER_TYPE_DEFAULT);
 			png_set_rows(png_ptr, info_ptr, p);
 			png_write_png(png_ptr, info_ptr,  PNG_TRANSFORM_IDENTITY, 0);
-			png_write_end(png_ptr, info_ptr);
+//			png_write_end(png_ptr, info_ptr);
 		}
 		png_destroy_write_struct(&png_ptr, &info_ptr);
 	}
