@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -24,7 +24,6 @@
 #include "mgl2/base.h"
 #include "interp.hpp"
 const double GAMMA=0.1;	///< value for damping
-HADT MGL_NO_EXPORT mglFormulaCalcC(const char *str, const std::vector<mglDataA*> &head);
 //-----------------------------------------------------------------------------
 //
 //		Advanced PDE series in 2D case
@@ -252,7 +251,7 @@ uintptr_t MGL_EXPORT mgl_pde_adv_(uintptr_t* gr, const char *ham, uintptr_t* ini
 //-----------------------------------------------------------------------------
 struct mgl_pde_ham
 {
-	dual *a,*hxy,*hxv,*huv,*huy;
+	ddual *a,*hxy,*hxv,*huv,*huy;
 	const char *eqs;
 	long nx,ny;
 	double xx,yy,xs,ys,dx,dy,dq,dp,zz;
@@ -315,19 +314,19 @@ HADT MGL_EXPORT mgl_pde_solve_c(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_
 	{	gr->SetWarn(mglWarnDim,"PDE");	return 0;	}
 	mglDataC *res=new mglDataC(nz, nx, ny);
 
-	dual *a = new dual[4*nx*ny], hh0;	// Add "damping" area
-	dual *hxy = new dual[4*nx*ny], *hxv = new dual[4*nx*ny];
-	dual *huy = new dual[4*nx*ny], *huv = new dual[4*nx*ny];
-	dual *hx = new dual[2*nx], *hv = new dual[2*ny];
-	dual *hy = new dual[2*ny], *hu = new dual[2*nx];
+	ddual *a = new ddual[4*nx*ny], hh0;	// Add "damping" area
+	ddual *hxy = new ddual[4*nx*ny], *hxv = new ddual[4*nx*ny];
+	ddual *huy = new ddual[4*nx*ny], *huv = new ddual[4*nx*ny];
+	ddual *hx = new ddual[2*nx], *hv = new ddual[2*ny];
+	ddual *hy = new ddual[2*ny], *hu = new ddual[2*nx];
 	double *dmp = new double[4*nx*ny];
-	memset(a,0,4*nx*ny*sizeof(dual));
+	memset(a,0,4*nx*ny*sizeof(ddual));
 	memset(dmp,0,4*nx*ny*sizeof(double));
 #pragma omp parallel for collapse(2)
 	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)	// Initial conditions
 	{
 		long i0 = i+nx/2+2*nx*(j+ny/2);
-		a[i0] = dual(ini_re->v(i,j), ini_im->v(i,j));
+		a[i0] = ddual(ini_re->v(i,j), ini_im->v(i,j));
 		res->a[nz*(i+nx*j)] = a[i0];
 	}
 #pragma omp parallel for collapse(2)
@@ -357,8 +356,8 @@ HADT MGL_EXPORT mgl_pde_solve_c(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_
 	{
 		if(gr->NeedStop())	break;
 		tmp.zz = Min.z+dz*k;
-		memset(hxy,0,4*nx*ny*sizeof(dual));	memset(hxv,0,4*nx*ny*sizeof(dual));
-		memset(huv,0,4*nx*ny*sizeof(dual));	memset(huy,0,4*nx*ny*sizeof(dual));
+		memset(hxy,0,4*nx*ny*sizeof(ddual));	memset(hxv,0,4*nx*ny*sizeof(ddual));
+		memset(huv,0,4*nx*ny*sizeof(ddual));	memset(huy,0,4*nx*ny*sizeof(ddual));
 		mgl_pde_hprep(&tmp);
 		for(long i=0;i<2*nx;i++)	{	hx[i] = hxv[i];			hu[i] = huv[i];		}
 		for(long j=0;j<2*ny;j++)	{	hy[j] = huy[2*nx*j];	hv[j] = huv[2*nx*j];}
@@ -446,15 +445,190 @@ uintptr_t MGL_EXPORT mgl_pde_solve_(uintptr_t* gr, const char *ham, uintptr_t* i
 //		ODE series
 //
 //-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_set_func(const mreal *x, mreal *dx, void *par)
+{
+	mglEqTxT *p=(mglEqTxT *)par;
+	long n = p->n/p->m;
+	for(long i=0;i<p->m;i++)
+	{
+		HMDT d = static_cast<HMDT>(p->head[i]);
+		memcpy(d->a, x+i*n, n*sizeof(mreal));
+	}
+	p->t->a[0] = x[p->n];
+//#pragma omp parallel for collapse(2)
+	for(long j=0;j<p->m;j++)
+	{
+		HMDT d = mglFormulaCalc(p->str[j].c_str(),p->head);
+		mreal val = d->a[0];
+		if(d->nx<n)
+		{
+			d->Create(n);
+			for(long i=0;i<n;i++)	d->a[i] = val;
+		}
+		else switch(p->brd)
+		{
+			default:	// zero instead of NAN
+			case 0:	
+			case 'z':
+			case '0':
+				for(long i=0;i<n;i++)	if(mgl_isbad(d->a[i]))	d->a[i] = 0;
+				break;
+			case 1:		// constant at border
+			case 'c':
+			case '1':
+				d->a[0] = d->a[1];	d->a[n-1] = d->a[n-2];	break;
+			case 2:		// linear at border
+			case 'l':
+			case '2':
+				d->a[0] = mreal(2)*d->a[1]-d->a[2];
+				d->a[n-1] = mreal(2)*d->a[n-2]-d->a[n-3];	break;
+			case 3:		// square at border
+			case 's':
+			case '3':
+				d->a[0] = d->a[3]+mreal(3)*(d->a[1]-d->a[2]);
+				d->a[n-1] = d->a[n-4]+mreal(3)*(d->a[n-2]-d->a[n-3]);	break;
+		}
+		memcpy(dx+j*n, d->a, n*sizeof(mreal));
+		delete d;
+	}
+}
+HMDT MGL_EXPORT mgl_ode_solve_set(const char *func, const char *var, char brd, HCDT x0, mreal dt, mreal tmax)
+{
+	if(!var || !(*var) || !func || !x0)	return 0;
+	mglEqTxT par;
+	par.var=var;	par.brd=brd;	par.FillStr(func);
+	const long n = par.n = x0->GetNx();
+	const long m = par.m = long(strlen(var)), nn = n/m;	// number of variables
+	
+	HMDT dat = new mglData[m+3];
+	for(long i=0;i<m;i++)
+	{
+		HMDT d = dat+i;	d->Create(nn);
+		d->s = var[i];	par.head.push_back(d);
+	}
+	HMDT d = dat+m+1;	d->Create(nn);
+	for(long i=0;i<nn;i++)	d->a[i] = i;
+	d->s = 'j';	par.head.push_back(d);
+	(dat+m+2)->Create(nn);	(dat+m+2)->s = "#$mgl";
+	par.head.push_back(dat+m+2);
+	par.t = dat+m;	par.t->s = 't';	par.head.push_back(par.t);
+	
+	mreal *xx = new mreal[n];
+	for(long i=0;i<n;i++)	xx[i] = x0->vthr(i);
+	HMDT res = mgl_ode_solve_ex(mgl_set_func,n,xx,dt,tmax,&par,NULL);
+	delete []xx;	delete []dat;	return res;
+}
+//-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_set_funcC(const mreal *x, mreal *dx, void *par)
+{
+	mglEqTxT *p=(mglEqTxT *)par;
+	long n = p->n/p->m;
+	for(long i=0;i<p->m;i++)
+	{
+		HADT d = static_cast<HADT>(p->head[i]);
+		memcpy(d->a, x+2*i*n, 2*n*sizeof(mreal));
+	}
+	p->t->a[0] = x[2*p->n];
+//#pragma omp parallel for collapse(2)
+	for(long j=0;j<p->m;j++)
+	{
+		HADT d = mglFormulaCalcC(p->str[j].c_str(),p->head);
+		dual val = d->a[0];
+		if(d->nx<n)
+		{
+			d->Create(n);
+			for(long i=0;i<n;i++)	d->a[i] = val;
+		}
+		else switch(p->brd)
+		{
+			default:
+			case 0:		// zero instead of NAN
+			case 'z':
+			case '0':
+				for(long i=0;i<n;i++)	if(mgl_isbad(d->a[i]))	d->a[i] = 0;
+				break;
+			case 1:		// constant at border
+			case 'c':
+			case '1':
+				d->a[0] = d->a[1];	d->a[n-1] = d->a[n-2];	break;
+			case 2:		// linear at border
+			case 'l':
+			case '2':
+				d->a[0] = mreal(2)*d->a[1]-d->a[2];
+				d->a[n-1] = mreal(2)*d->a[n-2]-d->a[n-3];	break;
+			case 3:		// square at border
+			case 's':
+			case '3':
+				d->a[0] = d->a[3]+mreal(3)*(d->a[1]-d->a[2]);
+				d->a[n-1] = d->a[n-4]+mreal(3)*(d->a[n-2]-d->a[n-3]);	break;
+			case -1:		// exponent at border
+			case 4:
+			case 'e':
+			case '4':
+				d->a[0] = norm(d->a[2])<norm(d->a[1]) ? d->a[1] : d->a[1]*d->a[1]/d->a[2];
+				d->a[n-1] = norm(d->a[n-3])<norm(d->a[n-2]) ? d->a[n-2] : d->a[n-2]*d->a[n-2]/d->a[n-3];
+				break;
+			case -2:		// gaussian at border
+			case 5:
+			case 'g':
+			case '5':
+				d->a[0] = norm(d->a[2])<norm(d->a[1]) ? d->a[3] : pow(d->a[1]/d->a[2],3)*d->a[3];
+				d->a[n-1] = norm(d->a[n-3])<norm(d->a[n-2]) ? d->a[n-4] : pow(d->a[n-2]/d->a[n-3],3)*d->a[n-4];
+				break;
+		}
+		memcpy(dx+2*j*n, d->a, 2*n*sizeof(mreal));
+		delete d;
+	}
+}
+HADT MGL_EXPORT mgl_ode_solve_set_c(const char *func, const char *var, char brd, HCDT x0, mreal dt, mreal tmax)
+{
+	if(!var || !(*var) || !func || !x0)	return 0;
+	mglEqTxT par;
+	par.var=var;	par.brd=brd;	par.FillStr(func);
+	const long n = par.n = x0->GetNx();
+	const long m = par.m = long(strlen(var)), nn = n/m;	// number of variables
+	
+	HADT dat = new mglDataC[m];
+	for(long i=0;i<m;i++)
+	{
+		HADT d = dat+i;	d->Create(nn);
+		d->s = var[i];	par.head.push_back(d);
+	}
+	HMDT d = new mglData[3];
+	d->Create(nn);	d->s = 'j';
+	for(long i=0;i<nn;i++)	d->a[i] = i;
+	par.head.push_back(d);
+	(d+1)->Create(nn);	(d+1)->s = "#$mgl";
+	par.head.push_back(d+1);
+	par.t = d+2;	par.t->s = 't';
+	par.head.push_back(par.t);
+
+	mreal *xx = new mreal[2*n];
+	const mglDataC *c = dynamic_cast<const mglDataC *>(x0);
+	if(c)		for(long i=0;i<n;i++)	
+	{	xx[2*i]=real(c->a[i]);	xx[2*i+1]=imag(c->a[i]);	}
+	else if(x0)	for(long i=0;i<n;i++)	
+	{	xx[2*i] = x0->vthr(i);	xx[2*i+1]=0;	}
+	else		for(long i=0;i<n;i++)	xx[2*i] = xx[2*i+1]=0;
+
+	HMDT res = mgl_ode_solve_ex(mgl_set_funcC,2*n,xx,dt,tmax,&par,NULL);
+	delete []xx;	delete []d;	delete []dat;
+	const long nt=res->ny;
+	mglDataC *out = new mglDataC(n, nt);
+#pragma omp parallel for
+	for(long i=0;i<nt*n;i++)	out->a[i] = dual(res->a[2*i],res->a[2*i+1]);
+	delete res;	return out;
+}
+//-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_txt_func(const mreal *x, mreal *dx, void *par)
 {
 	mglEqTxT *p=(mglEqTxT *)par;
 	mreal vars[MGL_VS];
-	size_t n = p->str.size();
-	for(size_t i=0;i<n;i++)
+	const long n = p->n;
+	for(long i=0;i<n;i++)
 	{	char ch = p->var[i];	if(ch>='a' && ch<='z')	vars[ch-'a']=x[i];	}
-#pragma omp parallel for
-	for(long i=0;i<long(n);i++)
+//#pragma omp parallel for
+	for(long i=0;i<n;i++)
 		dx[i] = mgl_expr_eval_v(p->eqR[i], vars);
 }
 HMDT MGL_EXPORT mgl_ode_solve_str(const char *func, const char *var, HCDT x0, mreal dt, mreal tmax)
@@ -462,9 +636,9 @@ HMDT MGL_EXPORT mgl_ode_solve_str(const char *func, const char *var, HCDT x0, mr
 	if(!var || !(*var) || !func)	return 0;
 	mglEqTxT par;
 	par.var=var;	par.FillReal(func);
-	size_t n = par.str.size();
+	long n = par.n = long(par.str.size());
 	mreal *xx = new mreal[n];
-	for(size_t i=0;i<n;i++)	xx[i] = x0?x0->vthr(i):0;
+	for(long i=0;i<n;i++)	xx[i] = x0?x0->vthr(i):0;
 	HMDT res = mgl_ode_solve_ex(mgl_txt_func,n,xx,dt,tmax,&par,NULL);
 	delete []xx;	return res;
 }
@@ -473,11 +647,11 @@ void MGL_NO_EXPORT mgl_txt_funcC(const mreal *x, mreal *dx, void *par)
 {
 	mglEqTxT *p=(mglEqTxT *)par;
 	mdual vars[MGL_VS];
-	size_t n = p->str.size();
-	for(size_t i=0;i<n;i++)
+	const long n = p->n;
+	for(long i=0;i<n;i++)
 	{	char ch = p->var[i];	if(ch>='a' && ch<='z')	vars[ch-'a']=dual(x[2*i],x[2*i+1]);	}
-#pragma omp parallel for
-	for(long i=0;i<long(n);i++)
+//#pragma omp parallel for
+	for(long i=0;i<n;i++)
 	{
 		dual r = mgl_cexpr_eval_v(p->eqC[i], vars);
 		dx[2*i] = real(r);	dx[2*i+1] = imag(r);
@@ -488,20 +662,20 @@ HADT MGL_EXPORT mgl_ode_solve_str_c(const char *func, const char *var, HCDT x0,
 	if(!var || !(*var) || !func)	return 0;
 	mglEqTxT par;	par.var=var;
 	par.var=var;	par.FillCmplx(func);
-	size_t n = par.str.size();
+	long n = par.n = long(par.str.size());
 	mreal *xx = new mreal[2*n];
 	const mglDataC *c = dynamic_cast<const mglDataC *>(x0);
-	for(size_t i=0;i<n;i++)
+	for(long i=0;i<n;i++)
 	{
 		if(c)	{	xx[2*i]=real(c->a[i]);	xx[2*i+1]=imag(c->a[i]);	}
 		else	{	xx[2*i] = x0?x0->vthr(i):0;	xx[2*i+1]=0;	}
 	}
 	HMDT res = mgl_ode_solve_ex(mgl_txt_funcC,2*n,xx,dt,tmax,&par,NULL);
 	delete []xx;
-	const long nn=n, nt=res->ny;
-	mglDataC *out = new mglDataC(nn, nt);
+	const long nt=res->ny;
+	mglDataC *out = new mglDataC(n, nt);
 #pragma omp parallel for
-	for(long i=0;i<nt*nn;i++)	out->a[i] = dual(res->a[2*i],res->a[2*i+1]);
+	for(long i=0;i<nt*n;i++)	out->a[i] = dual(res->a[2*i],res->a[2*i+1]);
 	delete res;	return out;
 }
 //-----------------------------------------------------------------------------
@@ -509,24 +683,60 @@ HMDT MGL_EXPORT mgl_ode_solve(void (*func)(const mreal *x, mreal *dx, void *par)
 {	return mgl_ode_solve_ex(func,n,x0,dt,tmax,par,0);	}
 HMDT MGL_EXPORT mgl_ode_solve_ex(void (*func)(const mreal *x, mreal *dx, void *par), int n, const mreal *x0, mreal dt, mreal tmax, void *par, void (*bord)(mreal *x, const mreal *xp, void *par))
 {
+	bool scale=false;
+	if(dt*tmax<0)	{	scale = true;	dt = fabs(dt);	tmax = fabs(tmax);	}
 	if(tmax<dt)	return 0;	// nothing to do
 	const long nt = int(tmax/dt+0.5)+1;
 	mglData *res=new mglData(n,nt);
-	mreal *x=new mreal[n], *k1=new mreal[n], *k2=new mreal[n], *k3=new mreal[n], *v=new mreal[n], hh=dt/2;
+	mreal *x=new mreal[n+1], *k1=new mreal[n], *k2=new mreal[n], *k3=new mreal[n], *v=new mreal[n+1], hh=dt/2;
 	// initial conditions
 	for(long i=0;i<n;i++)	x[i] = res->a[i] = x0[i];
 	// Runge Kutta scheme of 4th order
 	bool good=true;
 	long k;
+	x[n] = 0;
 	for(k=1;k<nt && good;k++)
 	{
+		double m0=0,m1=0,m2=0,m3=0, t = x[n];
 		func(x,k1,par);
+		if(scale)
+		{
+			for(long i=0;i<n;i++)
+			{	double kk = fabs(k1[i]);	if(kk>m0)	m0 = kk;	}
+			for(long i=0;i<n;i++)	k1[i] /= m0;
+		}
+		else m0 = 1;
 		for(long i=0;i<n;i++)	v[i] = x[i]+k1[i]*hh;
+		v[n] = t+hh/m0;
 		func(v,k2,par);
+		if(scale)
+		{
+			for(long i=0;i<n;i++)
+			{	double kk = fabs(k2[i]);	if(kk>m1)	m1 = kk;	}
+			for(long i=0;i<n;i++)	k2[i] /= m1;
+		}
+		else m1 = 1;
 		for(long i=0;i<n;i++)	v[i] = x[i]+k2[i]*hh;
+		v[n] = t+hh/m1;
 		func(v,k3,par);
+		if(scale)
+		{
+			for(long i=0;i<n;i++)
+			{	double kk = fabs(k3[i]);	if(kk>m2)	m2 = kk;	}
+			for(long i=0;i<n;i++)	k3[i] /= m2;
+		}
+		else m2 = 1;
 		for(long i=0;i<n;i++)	{	v[i] = x[i]+k3[i]*dt;	k3[i] += k2[i];	}
+		v[n] = t+dt/m2;
 		func(v,k2,par);
+		if(scale)
+		{
+			for(long i=0;i<n;i++)
+			{	double kk = fabs(k2[i]);	if(kk>m3)	m3 = kk;	}
+			for(long i=0;i<n;i++)	k2[i] /= m3;
+		}
+		else m3 = 1;
+		x[n] = t + (1/m0+2/m1+2/m2+1/m3)*dt/6;
 		for(long i=0;i<n;i++)	x[i] += (k1[i]+k2[i]+2*k3[i])*dt/6;
 		if(bord)	bord(x,res->a+n*(k-1),par);
 		for(long i=0;i<n;i++)
@@ -658,11 +868,11 @@ static void *mgl_qo2d_hprep(void *par)
 		hh = sqrt(sqrt(0.041+hh*hh*hh*hh));
 		mreal tt = (ra->pt + ra->d1*x1)/hh - ra->pt;
 		dual tmp = f->ham(abs(f->a[i]), r[0]+ra->x1*x1, r[1]+ra->y1*x1, r[3]+ra->x0*tt, r[4]+ra->y0*tt, f->par);
-		f->hx[i] = tmp - f->h0/2.;
+		f->hx[i] = tmp - f->h0/mreal(2);
 		// u-y terms
 		x1 = f->dk/2*(i<nx/2 ? i:i-nx);
 		tmp = f->ham(0, r[0], r[1], r[3]+ra->x1*x1, r[4]+ra->y1*x1, f->par);
-		f->hu[i] = tmp - f->h0/2.;
+		f->hu[i] = tmp - f->h0/mreal(2);
 
 		if(imag(f->hx[i])>0)	f->hx[i] = f->hx[i].real();
 		if(imag(f->hu[i])>0)	f->hu[i] = f->hu[i].real();
@@ -702,7 +912,7 @@ HADT MGL_EXPORT mgl_qo2d_func_c(mdual (*ham)(mreal u, mreal x, mreal y, mreal px
 	for(long k=0;k<nt;k++)
 	{
 		for(long i=0;i<nx;i++)	// "save"
-			res->a[i+k*nx]=a[i+nx/2]*sqrt(ra[0].ch/ra[k].ch);
+			res->a[i+k*nx]=a[i+nx/2]*mreal(sqrt(ra[0].ch/ra[k].ch));
 		if(xx && yy)	for(long i=0;i<nx;i++)	// prepare xx, yy
 		{
 			mreal x1 = (2*i-nx+1)*dr;
@@ -841,13 +1051,13 @@ static void *mgl_qo3d_post(void *par)
 	for(long ii=t->id;ii<nx*nx;ii+=mglNumThr)
 	{
 		long i = ii%nx, j = ii/nx;
-		f->hxy[ii] -= (f->hx[i]+f->hy[j]-f->h0/2.)/2.;
+		f->hxy[ii] -= (f->hx[i]+f->hy[j]-f->h0/mreal(2))/mreal(2);
 		if(imag(f->hxy[ii])>0)	f->hxy[ii] = f->hxy[ii].real();
-		f->hxv[ii] -= (f->hx[i]+f->hv[j]-f->h0/2.)/2.;
+		f->hxv[ii] -= (f->hx[i]+f->hv[j]-f->h0/mreal(2))/mreal(2);
 		if(imag(f->hxv[ii])>0)	f->hxv[ii] = f->hxv[ii].real();
-		f->huy[ii] -= (f->hu[i]+f->hy[j]-f->h0/2.)/2.;
+		f->huy[ii] -= (f->hu[i]+f->hy[j]-f->h0/mreal(2))/mreal(2);
 		if(imag(f->huy[ii])>0)	f->huy[ii] = f->huy[ii].real();
-		f->huv[ii] -= (f->hu[i]+f->hv[j]-f->h0/2.)/2.;
+		f->huv[ii] -= (f->hu[i]+f->hv[j]-f->h0/mreal(2))/mreal(2);
 		if(imag(f->huv[ii])>0)	f->huv[ii] = f->huv[ii].real();
 		// add boundary conditions for x-direction
 		f->hxy[ii] -= dual(0,f->dmp[ii]);
@@ -895,7 +1105,7 @@ HADT MGL_EXPORT mgl_qo3d_func_c(mdual (*ham)(mreal u, mreal x, mreal y, mreal z,
 	{
 #pragma omp parallel for collapse(2)
 		for(long i=0;i<nx;i++)	for(long j=0;j<nx;j++)	// "save"
-			res->a[i+nx*(j+k*nx)]=a[i+nx/2+2*nx*(j+nx/2)]*sqrt(ra[0].ch/ra[k].ch);
+			res->a[i+nx*(j+k*nx)]=a[i+nx/2+2*nx*(j+nx/2)]*mreal(sqrt(ra[0].ch/ra[k].ch));
 		if(xx && yy && zz)
 #pragma omp parallel for collapse(2)
 			for(long i=0;i<nx;i++)	for(long j=0;j<nx;j++)	// prepare xx, yy, zz
@@ -1221,57 +1431,65 @@ HMDT MGL_EXPORT mgl_data_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *how
 	if(mglchr(how,'x') && (na==nn || na==np || na==nx))
 #pragma omp parallel
 	{
-		mglData T(nx,4);	mreal *uu=T.a+(per?3:2)*nx;
+		mreal *tt = new mreal[4*nx], *uu=tt+(per?3:2)*nx;
+// 		mglData T(nx,4);	mreal *uu=T.a+(per?3:2)*nx;
 #pragma omp for collapse(2)
 		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)
 		{
 			long i0=0, i1=nx*(j+ny*k);
 			if(na==nn)	i0=nx*(j+ny*k);	else if(na==np)	i0=nx*j;
-			if(per)	mgl_progonka_pr(A,B,C,D,T.a,nx,i1,i0,1,difr);
-			else	mgl_progonka_sr(A,B,C,D,T.a,nx,i1,i0,1,difr);
+			if(per)	mgl_progonka_pr(A,B,C,D,tt,nx,i1,i0,1,difr);
+			else	mgl_progonka_sr(A,B,C,D,tt,nx,i1,i0,1,difr);
 			i0 = nx*(j+ny*k);
 			for(long i=0;i<nx;i++)	r->a[i+i0] = uu[i];
 		}
+		delete []tt;
 	}
 	else if(mglchr(how,'y') && (na==nn || na==np || na==ny))
 #pragma omp parallel
 	{
-		mglData T(ny,4);	mreal *uu=T.a+(per?3:2)*ny;
+		mreal *tt = new mreal[4*ny], *uu=tt+(per?3:2)*ny;
+// /		mglData T(ny,4);	mreal *uu=T.a+(per?3:2)*ny;
 #pragma omp for collapse(2)
 		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)
 		{
 			long i0=0, i1 = i+np*k;
 			if(na==nn)	i0=i+np*k;	else if(na==np)	i0=i;
-			if(per)	mgl_progonka_pr(A,B,C,D,T.a,ny,i1,i0,nx,difr);
-			else	mgl_progonka_sr(A,B,C,D,T.a,ny,i1,i0,nx,difr);
+			if(per)	mgl_progonka_pr(A,B,C,D,tt,ny,i1,i0,nx,difr);
+			else	mgl_progonka_sr(A,B,C,D,tt,ny,i1,i0,nx,difr);
 			i0 = i+np*k;
 			for(long j=0;j<ny;j++)	r->a[j*nx+i0] = uu[j];
 		}
+		delete []tt;
 	}
 	else if(mglchr(how,'z') && (na==nn || na==nz))
 #pragma omp parallel
 	{
-		mglData T(nz,4);	mreal *uu=T.a+(per?3:2)*nz;
+		mreal *tt = new mreal[4*nz], *uu=tt+(per?3:2)*nz;
+// 		mglData T(nz,4);	mreal *uu=T.a+(per?3:2)*nz;
 #pragma omp for collapse(2)
 		for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 		{
 			long i0 = na==nn?i+nx*j:0, i1 = i+nx*j;
-			if(per)	mgl_progonka_pr(A,B,C,D,T.a,nz,i1,i0,np,difr);
-			else	mgl_progonka_sr(A,B,C,D,T.a,nz,i1,i0,np,difr);
+			if(per)	mgl_progonka_pr(A,B,C,D,tt,nz,i1,i0,np,difr);
+			else	mgl_progonka_sr(A,B,C,D,tt,nz,i1,i0,np,difr);
 			i0 = i+nx*j;
 			for(long k=0;k<nz;k++)	r->a[k*np+i0] = uu[k];
 		}
+		delete []tt;
 	}
 	else if(mglchr(how,'h') && ny==nx && (na==nn || na==np) && nx>1)
 #pragma omp parallel
 	{
-		mglData T(np,2);
+		mreal *tt = new mreal[2*np];
+// 		mglData T(np,2);
 #pragma omp for
 		for(long k=0;k<nz;k++)
 		{
-			mgl_progonka_hr(A,B,C,D,T.a,nx,k*np,na==nn ? k*np:0,difr);
-			memcpy(r->a+k*np, T.a+np, np*sizeof(mreal));
+			mgl_progonka_hr(A,B,C,D,tt,nx,k*np,na==nn ? k*np:0,difr);
+			memcpy(r->a+k*np, tt+np, np*sizeof(mreal));
 		}
+		delete []tt;
 	}
 	else	{	delete r;	r=0;	}
 	return r;
@@ -1381,55 +1599,63 @@ HADT MGL_EXPORT mgl_datac_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *ho
 	if(mglchr(how,'x') && (na==nn || na==np || na==nx))
 #pragma omp parallel
 	{
-		mglDataC T(nx,4);	dual *uu=T.a+(per?3:2)*nx;
+		dual *tt = new dual[4*nx], *uu=tt+(per?3:2)*nx;
+// 		mglDataC T(nx,4);	dual *uu=T.a+(per?3:2)*nx;
 #pragma omp for collapse(2)
 		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)
 		{
 			long i0=0, i1=nx*(j+ny*k);
 			if(na==nn)	i0=i1;	else if(na==np)	i0=nx*j;
-			if(per)	mgl_progonka_pc(A,B,C,D,T.a,nx,i1,i0,1,difr);
-			else	mgl_progonka_sc(A,B,C,D,T.a,nx,i1,i0,1,difr);
+			if(per)	mgl_progonka_pc(A,B,C,D,tt,nx,i1,i0,1,difr);
+			else	mgl_progonka_sc(A,B,C,D,tt,nx,i1,i0,1,difr);
 			for(long i=0;i<nx;i++)	r->a[i+i1] = uu[i];
 		}
+		delete []tt;
 	}
 	else if(mglchr(how,'y') && (na==nn || na==np || na==ny))
 #pragma omp parallel
 	{
-		mglDataC T(ny,4);	dual *uu=T.a+(per?3:2)*ny;
+		dual *tt = new dual[4*ny], *uu=tt+(per?3:2)*ny;
+// 		mglDataC T(ny,4);	dual *uu=T.a+(per?3:2)*ny;
 #pragma omp for collapse(2)
 		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)
 		{
 			long i0=0, i1 = i+np*k;
 			if(na==nn)	i0=i1;	else if(na==np)	i0=i;
-			if(per)	mgl_progonka_pc(A,B,C,D,T.a,ny,i1,i0,nx,difr);
-			else	mgl_progonka_sc(A,B,C,D,T.a,ny,i1,i0,nx,difr);
+			if(per)	mgl_progonka_pc(A,B,C,D,tt,ny,i1,i0,nx,difr);
+			else	mgl_progonka_sc(A,B,C,D,tt,ny,i1,i0,nx,difr);
 			i0 = i+np*k;
 			for(long j=0;j<ny;j++)	r->a[j*nx+i0] = uu[j];
 		}
+		delete []tt;
 	}
 	else if(mglchr(how,'z') && (na==nn || na==nz))
 #pragma omp parallel
 	{
-		mglDataC T(nz,4);	dual *uu=T.a+(per?3:2)*nz;
+		dual *tt = new dual[4*nz], *uu=tt+(per?3:2)*ny;
+// 		mglDataC T(nz,4);	dual *uu=T.a+(per?3:2)*nz;
 #pragma omp for collapse(2)
 		for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 		{
 			long i0 = na==nn?i+nx*j:0, i1 = i+nx*j;
-			if(per)	mgl_progonka_pc(A,B,C,D,T.a,nz,i1,i0,np,difr);
-			else	mgl_progonka_sc(A,B,C,D,T.a,nz,i1,i0,np,difr);
+			if(per)	mgl_progonka_pc(A,B,C,D,tt,nz,i1,i0,np,difr);
+			else	mgl_progonka_sc(A,B,C,D,tt,nz,i1,i0,np,difr);
 			for(long k=0;k<nz;k++)	r->a[k*np+i1] = uu[k];
 		}
+		delete []tt;
 	}
 	else if(mglchr(how,'h') && ny==nx && (na==nn || na==np) && nx>1)
 #pragma omp parallel
 	{
-		mglDataC T(np,2);
+		dual *tt = new dual[2*np];
+// 		mglDataC T(np,2);
 #pragma omp for
 		for(long k=0;k<nz;k++)
 		{
-			mgl_progonka_hc(A,B,C,D,T.a,nx,k*np, na==nn ? k*np:0,difr);
-			memcpy(r->a+k*np, T.a+np, np*sizeof(dual));
+			mgl_progonka_hc(A,B,C,D,tt,nx,k*np, na==nn ? k*np:0,difr);
+			memcpy(r->a+k*np, tt+np, np*sizeof(dual));
 		}
+		delete []tt;
 	}
 	else	{	delete r;	r=0;	}
 	return r;
