@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -79,7 +79,7 @@ void mglCanvas::SetSize(int w,int h,bool clf)
 #pragma omp parallel for
 	for(long i=0;i<s;i++)
 	{	unsigned char *b=GB+4*i;
-		b[0]=BDef[0];	b[1]=BDef[1];	b[2]=BDef[2];	b[3]=BDef[3];	}
+		b[0]=BDef[0];	b[1]=BDef[1];	b[2]=BDef[2];	b[3]=0/*BDef[3]*/;	}
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_unlock(&mutexClf);
 #elif MGL_HAVE_OMP
@@ -316,7 +316,7 @@ void mglCanvas::CalcScr(mglPoint p, int *xs, int *ys) const
 	if(ys)	*ys=int(p.y);
 }
 //-----------------------------------------------------------------------------
-mglPoint mglCanvas::CalcScr(mglPoint p) const
+mglPoint mglCanvas::CalcScr(const mglPoint &p) const
 {	int x,y;	CalcScr(p,&x,&y);	return mglPoint(x,y);	}
 //-----------------------------------------------------------------------------
 /*void static mgl_prm_swap(mglPrim &s1,mglPrim &s2,mglPrim *buf)
@@ -473,6 +473,8 @@ void mglCanvas::Finish()
 		mglStartThread(&mglCanvas::pxl_memcpy,this,n);
 	}
 	int x2 = BBoxX2<0?Width:BBoxX2, y2 = BBoxY2<0?Height:BBoxY2;
+	if(x2>Width)	x2=Width;
+	if(y2>Height)	y2=Height;
 	if(BBoxX1>=0 && BBoxX1<x2 && BBoxY1>=0 && BBoxY1<y2)
 	{
 		unsigned char ff[8]={255,255,255,255, 0,0,0,255}, *g1 = G4+BBoxX1*4-4;
@@ -550,6 +552,73 @@ void mglCanvas::LoadBackground(const char *fname, double alpha)
 		for(long i=0;i<Width*Height;i++)	GB[4*i+3] = (unsigned char)(GB[4*i+3]*alpha);
 }
 //-----------------------------------------------------------------------------
+void mglCanvas::LoadBackground(const char *fname, const char *how, double alpha)
+{
+	int w,h;	// bitmap sizes
+	unsigned char *g=NULL;
+	mgl_read_image(&g,w,h,fname);
+	if(!g)	return;
+	if(alpha<1 && alpha>0)
+#pragma omp parallel for
+		for(long i=0;i<w*h;i++)	g[4*i+3] = (unsigned char)(g[4*i+3]*alpha);
+	int ww=Width, hh=Height, ii=0, jj=0;	// canvas range
+	if(mglchr(how,'a'))	{	ww=inW;	hh=inH;	ii=inX;	jj=Height-inY-inH;	}
+	int di = 0, dj=0;	// bitmap shift
+	bool center=false;
+	if(mglchr(how,'c'))	{	center=true;	di = (w-ww)/2;	dj = (h-hh)/2;	}
+	if(mglchr(how,'s'))	// spline resizing
+	{
+		double ki=double(w-1)/(ww-1), kj=double(h-1)/(hh-1);
+		for(int j=0;j<hh-1;j++)
+		{
+			double tj = kj*j;
+			int pj = int(tj);
+			if(pj>h-1)	{	pj--;	tj=1;	}	else	tj -= pj;
+			for(int i=0;i<ww-1;i++)
+			{
+				unsigned char *b=GB+4*(ii+i+Width*(jj+j));
+				int pi = int(ki*i);
+				double ti = ki*i-pi;
+				const unsigned char *p=g+4*(pi+w*pj), *q=g+4*(pi+w*pj+w);
+				b[0] = p[0]*(1-ti)*(1-tj) + p[4]*ti*(1-tj) + q[0]*(1-ti)*tj + q[4]*ti*tj;
+				b[1] = p[1]*(1-ti)*(1-tj) + p[5]*ti*(1-tj) + q[1]*(1-ti)*tj + q[5]*ti*tj;
+				b[2] = p[2]*(1-ti)*(1-tj) + p[6]*ti*(1-tj) + q[2]*(1-ti)*tj + q[6]*ti*tj;
+				b[3] = p[3]*(1-ti)*(1-tj) + p[7]*ti*(1-tj) + q[3]*(1-ti)*tj + q[7]*ti*tj;
+			}
+			unsigned char *b=GB+4*(ii+ww-1+Width*(jj+j));
+			const unsigned char *p=g+4*(w*(pj+1)-1), *q=g+4*(w*(pj+2)-1);
+			b[0] = q[0]*tj + p[0]*(1-tj);
+			b[1] = q[1]*tj + p[1]*(1-tj);
+			b[2] = q[2]*tj + p[2]*(1-tj);
+			b[3] = q[3]*tj + p[3]*(1-tj);
+		}
+	}
+	else if(mglchr(how,'m'))	// mosaic
+	{
+		while(di<0)	di+=w;	// stupid correction
+		while(dj<0)	dj+=h;
+		for(int j=0;j<hh;j++)
+		{
+			int pj = (j+dj)%h;
+			for(int i=0;i<ww;i++)
+			{
+				unsigned char *b=GB+4*(ii+i+Width*(jj+j));
+				int pi = (i+di)%w;
+				unsigned char *p=g+4*(pi+w*pj);
+				b[0] = p[0];	b[1] = p[1];	b[2] = p[2];	b[3] = p[3];
+			}
+		}
+	}
+	else
+	{
+		if(w<ww)	{	ww=w;	if(center)	{	ii-=di;	di=0;	}	}
+		if(h<hh)	{	hh=h;	if(center)	{	jj-=dj;	dj=0;	}	}
+		for(int j=0;j<hh;j++)
+			memcpy(GB+4*(ii+Width*(jj+j)), g+4*(di+w*(dj+j)), 4*ww);
+	}
+	if(g)	delete []g;
+}
+//-----------------------------------------------------------------------------
 void mglCanvas::FillBackground(const mglColor &cc)
 {
 	BDef[0] = (unsigned char)(255*cc.r);	BDef[1] = (unsigned char)(255*cc.g);
@@ -573,6 +642,8 @@ unsigned char **mglCanvas::GetRGBLines(long &w, long &h, unsigned char *&f, bool
 	long c = alpha?4:3, d = c*Width;
 	unsigned char *gg = (alpha?G4:G);
 	int x2 = BBoxX2<0?Width:BBoxX2, y2 = BBoxY2<0?Height:BBoxY2;
+	if(x2>Width)	x2=Width;
+	if(y2>Height)	y2=Height;
 	if(BBoxX1>=0 && BBoxX1<x2 && BBoxY1>=0 && BBoxY1<y2)
 	{
 		gg += c*BBoxX1 + d*BBoxY1;
