@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -212,7 +212,7 @@ void MGL_EXPORT mgl_candle_xyv(HMGL gr, HCDT x, HCDT v1, HCDT v2, HCDT y1, HCDT
 	{	if(d1)	delete y1;	if(d2)	delete y2;
 		gr->SetWarn(mglWarnDim,"Candle");	return;	}
 	static int cgid=1;	gr->StartGroup("Candle",cgid++);
-	gr->SaveState(opt);	gr->SetPenPal(pen,&pal);
+	gr->SaveState(opt);	gr->SetPenPal(pen,&pal);	gr->SetMask(pen);
 	long kq = gr->AllocPnts(8*n);
 	bool sh = mglchr(pen,'!');
 	bool wire = mglchr(pen,'#');
@@ -505,7 +505,8 @@ void MGL_EXPORT mgl_area_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 	bool sh = mglchr(pen,'!'), wire = mglchr(pen,'#'), orig = !mglchr(pen,'a');
 
 	double z0=gr->GetOrgZ('x');
-	gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
+	gr->SetPenPal(pen,&pal);	gr->SetMask(pen);	gr->Reserve(2*n*m);
+
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
@@ -556,7 +557,7 @@ void MGL_EXPORT mgl_area_xy(HMGL gr, HCDT x, HCDT y, const char *pen, const char
 	mglPoint nn(0,0,1);
 	bool sh = mglchr(pen,'!'), wire = mglchr(pen,'#'), orig = !mglchr(pen,'a');
 
-	gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
+	gr->SetPenPal(pen,&pal);	gr->SetMask(pen);	gr->Reserve(2*n*m);
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
@@ -697,7 +698,7 @@ void MGL_EXPORT mgl_region_3d(HMGL gr, HCDT x1, HCDT y1, HCDT z1, HCDT x2, HCDT
 //	bool inside = (mglchr(pen,'i'));	// NOTE: check if 'i' is free (used here for inside flag)
 	bool sh = mglchr(pen,'!'), wire = mglchr(pen,'#'), orig = !mglchr(pen,'a');
 
-	gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
+	gr->SetPenPal(pen,&pal);	gr->SetMask(pen);	gr->Reserve(2*n*m);
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
@@ -761,7 +762,7 @@ void MGL_EXPORT mgl_region_xy(HMGL gr, HCDT x, HCDT y1, HCDT y2, const char *pen
 	bool inside = mglchr(pen,'i');	// NOTE: check if 'i' is free (used here for inside flag)
 	bool sh = mglchr(pen,'!'), wire = mglchr(pen,'#'), orig = !mglchr(pen,'a');
 
-	gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
+	gr->SetPenPal(pen,&pal);	gr->SetMask(pen);	gr->Reserve(2*n*m);
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
@@ -969,6 +970,92 @@ void MGL_EXPORT mgl_step_(uintptr_t *gr, uintptr_t *y,	const char *pen, const ch
 //	Stem series
 //
 //-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_lines_xyz(HMGL gr, HCDT x1, HCDT y1, HCDT z1, HCDT x2, HCDT y2, HCDT z2, const char *pen, const char *opt)
+{
+	long m,n=y1->GetNx(), pal;
+	if(mgl_check_dim1(gr,x1,y1,z1,x2,"Lines"))	return;
+	if(mgl_check_dim1(gr,x2,y2,z2,NULL,"Lines"))	return;
+	if(x1->GetNy()!=x2->GetNy() || y1->GetNy()!=y2->GetNy() || z1->GetNy()!=z2->GetNy())	return;
+
+	gr->SaveState(opt);
+	static int cgid=1;	gr->StartGroup("Lines",cgid++);
+	m = x1->GetNy() > y1->GetNy() ? x1->GetNy() : y1->GetNy();	m = z1->GetNy() > m ? z1->GetNy() : m;
+	bool sh = mglchr(pen,'!');
+
+	gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
+	for(long j=0;j<m;j++)
+	{
+		if(gr->NeedStop())	break;
+		long mx = j<x1->GetNy() ? j:0, my = j<y1->GetNy() ? j:0, mz = j<z1->GetNy() ? j:0;
+		double c1=gr->NextColor(pal), c2=c1;
+		if(gr->GetNumPal(pal)==2*m && !sh)	c2=gr->NextColor(pal);
+		long kq = gr->AllocPnts(2*n);
+#pragma omp parallel for
+		for(long i=0;i<n;i++)
+		{
+			double cc=gr->NextColor(pal,i);
+			gr->AddPntQ(kq+2*i,mglPoint(x1->v(i,mx), y1->v(i,my), z1->v(i,mz)),sh?cc:c1);
+			gr->AddPntQ(kq+2*i+1,mglPoint(x2->v(i,mx), y2->v(i,my), z2->v(i,mz)),sh?cc:c2);
+		}
+		for(long i=0;i<n;i++)
+		{
+			long iq = kq+2*i;	gr->line_plot(iq,iq+1);
+			gr->arrow_plot(iq,iq+1,gr->Arrow2);
+			gr->arrow_plot(iq+1,iq,gr->Arrow1);
+		}
+	}
+	gr->EndGroup();
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_lines_xy(HMGL gr, HCDT x1, HCDT y1, HCDT x2, HCDT y2, const char *pen, const char *opt)
+{
+	gr->SaveState(opt);
+	mglDataV z(y1->GetNx());	z.Fill(gr->Min.z,gr->Min.z);
+	mgl_lines_xyz(gr,x1,y1,&z,x2,y2,&z,pen,0);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_lines_x(HMGL gr, HCDT x1, HCDT x2, const char *pen, const char *opt)
+{
+	gr->SaveState(opt);
+	mglDataV y(x1->GetNx()), z(x1->GetNx());
+	y.Fill(gr->Min.y,gr->Max.y);
+	z.Fill(gr->Min.z,gr->Min.z);
+	mgl_lines_xyz(gr,x1,&y,&z,x2,&y,&z,pen,0);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_lines(HMGL gr, HCDT y1, HCDT y2, const char *pen, const char *opt)
+{
+	gr->SaveState(opt);
+	mglDataV x(y1->GetNx()), z(y1->GetNx());
+	x.Fill(gr->Min.x,gr->Max.x);
+	z.Fill(gr->Min.z,gr->Min.z);
+	mgl_lines_xyz(gr,&x,y1,&z,&x,y2,&z,pen,0);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_lines_xyz_(uintptr_t *gr, uintptr_t *x1, uintptr_t *y1, uintptr_t *z1, uintptr_t *x2, uintptr_t *y2, uintptr_t *z2, const char *pen, const char *opt,int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,pen,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_lines_xyz(_GR_,_DA_(x1),_DA_(y1),_DA_(z1),_DA_(x2),_DA_(y2),_DA_(z2),s,o);	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_lines_xy_(uintptr_t *gr, uintptr_t *x1, uintptr_t *y1, uintptr_t *x2, uintptr_t *y2, const char *pen, const char *opt,int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,pen,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_lines_xy(_GR_,_DA_(x1),_DA_(y1),_DA_(x2),_DA_(y2),s,o);	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_lines_x_(uintptr_t *gr, uintptr_t *x1,	uintptr_t *x2,	const char *pen, const char *opt,int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,pen,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_lines_x(_GR_,_DA_(x1),_DA_(x2),s,o);	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_lines_(uintptr_t *gr, uintptr_t *y1,	uintptr_t *y2,	const char *pen, const char *opt,int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,pen,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_lines(_GR_,_DA_(y1),_DA_(y2),s,o);	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
+//
+//	Stem series
+//
+//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_stem_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, const char *opt)
 {
 	long m,n=y->GetNx(), pal;
@@ -1096,8 +1183,7 @@ void MGL_EXPORT mgl_bars_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 	}
 	if(dc==0)	fixed=false;	// NOTE: disable fixed width if it is zero
 
-	gr->SetPenPal(pen,&pal);
-	gr->Reserve(4*n*m);
+	gr->SetPenPal(pen,&pal);	gr->SetMask(pen);	gr->Reserve(4*n*m);
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
@@ -1181,8 +1267,7 @@ void MGL_EXPORT mgl_bars_xy(HMGL gr, HCDT x, HCDT y, const char *pen, const char
 	}
 	if(dx==0)	fixed=false;	// NOTE: disable fixed width if it is zero
 
-	gr->SetPenPal(pen,&pal);
-	gr->Reserve(4*n*m);
+	gr->SetPenPal(pen,&pal);	gr->SetMask(pen);	gr->Reserve(4*n*m);
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
@@ -1287,8 +1372,7 @@ void MGL_EXPORT mgl_barh_yx(HMGL gr, HCDT y, HCDT v, const char *pen, const char
 	}
 	if(dy==0)	fixed=false;	// NOTE: disable fixed width if it is zero
 
-	gr->SetPenPal(pen,&pal);
-	gr->Reserve(4*n*m);
+	gr->SetPenPal(pen,&pal);	gr->SetMask(pen);	gr->Reserve(4*n*m);
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
@@ -1765,7 +1849,7 @@ void MGL_EXPORT mgl_error_exy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintpt
 //		Chart series
 //
 //-----------------------------------------------------------------------------
-void face_plot(mglBase *gr, mglPoint o, mglPoint d1, mglPoint d2, double c, bool wire)
+void face_plot(mglBase *gr, const mglPoint &o, mglPoint d1, mglPoint d2, double c, bool wire)
 {
 	const int num=10;
 	mglPoint nn=d1^d2;
@@ -1818,6 +1902,7 @@ void MGL_EXPORT mgl_chart(HMGL gr, HCDT a, const char *cols, const char *opt)
 
 	double dy = (gr->Max.y-gr->Min.y)/a->GetNy(), dx, ss, cs, x1, y1, dz=gr->Max.z-gr->Min.z, vv;
 	mglPoint d1,d2,o;
+	gr->SetMask(cols);
 
 	for(j=0;j<a->GetNy();j++)
 	{
@@ -2064,7 +2149,7 @@ void MGL_EXPORT mgl_tape_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 	double rr = gr->SaveState(opt);
 	if(rr==0 || mgl_isnan(rr))	rr = mgl_norm(gr->Max-gr->Min)*gr->BarWidth/25;
 	m = x->GetNy() > y->GetNy() ? x->GetNy() : y->GetNy();	m = z->GetNy() > m ? z->GetNy() : m;
-	gr->SetPenPal(pen,&pal);	gr->Reserve(4*n*m);
+	gr->SetPenPal(pen,&pal);	gr->SetMask(pen);	gr->Reserve(4*n*m);
 	mglPoint qn(NAN,NAN);
 	bool sh = mglchr(pen,'!'), xo = mglchr(pen,'x'), zo = mglchr(pen,'z'), wire = mglchr(pen,'#');
 	if(!xo && !zo)	xo = zo = true;
