@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -375,7 +375,7 @@ void QMathGL::drawPrim()
 	for(i=0;i<n;i++)
 	{
 		mgl_set_obj_id(gr,i+MGL_MAX_LINES);
-		QString tst = primitives.section('\n',i,i);
+//		QString tst = primitives.section('\n',i,i);
 		pr.Parse(&gg,primitives.section('\n',i,i).toLocal8Bit().constData(),i+MGL_MAX_LINES);
 	}
 	gg.SetRanges(ox1,ox2);	gg.Pop();	setlocale(LC_NUMERIC, loc.c_str());
@@ -439,7 +439,7 @@ void QMathGL::mousePressEvent(QMouseEvent *ev)
 
 		p = gr->CalcXYZ(ev->x(), ev->y(), true);
 		if(mgl_isnan(p.x))	mousePos = "";
-		else	mousePos.sprintf("x=%g, y=%g, z=%g",p.x,p.y,p.z);
+		else	mousePos.asprintf("x=%g, y=%g, z=%g",p.x,p.y,p.z);
 		emit posChanged(mousePos);
 		repaint();
 	}
@@ -625,7 +625,7 @@ void QMathGL::mouseMoveEvent(QMouseEvent *ev)
 	{
 		mglPoint p = gr->CalcXYZ(ev->x(), ev->y(), true);
 		if(mgl_isnan(p.x))	mousePos = "";
-		else	mousePos.sprintf("x=%g, y=%g, z=%g",p.x,p.y,p.z);
+		else	mousePos.asprintf("x=%g, y=%g, z=%g",p.x,p.y,p.z);
 		emit posChanged(mousePos);
 	}*/
 	ev->accept();
@@ -971,7 +971,7 @@ void QMathGL::addText(QString txt)
 mglCanvasQT::mglCanvasQT() : mglCanvasWnd()
 {	Wnd = 0;	}
 mglCanvasQT::~mglCanvasQT()
-{	if(Wnd)	{	QMGL->gr=0;	delete Wnd;	}	}
+{	if(Wnd)	{	QMGL->noGraph();	delete Wnd;	}	}
 //-----------------------------------------------------------------------------
 void mglCanvasQT::GotoFrame(int d)
 {
